Package: statgenMPP
Title: QTL Mapping for Multi Parent Populations
Version: 1.0.5
Date: 2026-05-18
Authors@R: c(person(given = "Martin", 
                    family = "Boer", 
                    email = "martin.boer@wur.nl", 
                    role = "aut",
                    comment = c(ORCID = "0000-0002-1879-4588")),
             person(given = "Wenhao", 
                    family = "Li", 
                    role = "aut",
                    comment = c(ORCID = "0000-0001-5719-5775")),        
             person(given = "Bart-Jan", 
                    family = "van Rossum", 
                    email = "bart-jan.vanrossum@wur.nl", 
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-8673-2514")),
             person(given = "Fred", 
                    family = "van Eeuwijk",
                    role = "ctb",
                    comment = c(ORCID = "0000-0003-3672-2921")))        
Description: For Multi Parent Populations (MPP) Identity By Descend (IBD) 
    probabilities are computed using Hidden Markov Models. These probabilities 
    are then used in a mixed model approach for QTL Mapping as described in 
    Li et al. (<doi:10.1007/s00122-021-03919-7>).
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.6), statgenGWAS (>= 1.0.9)
Imports: data.table, foreach, ggplot2, gridExtra, LMMsolver (>= 1.0.5),
        rlang, scales, spam, statgenIBD (>= 1.0.8)
Suggests: rmarkdown, knitr, tinytest
VignetteBuilder: knitr
URL: https://biometris.github.io/statgenMPP/index.html,
        https://github.com/Biometris/statgenMPP/
BugReports: https://github.com/Biometris/statgenMPP/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 12:54:26 UTC; rossu027
Author: Martin Boer [aut] (ORCID: <https://orcid.org/0000-0002-1879-4588>),
  Wenhao Li [aut] (ORCID: <https://orcid.org/0000-0001-5719-5775>),
  Bart-Jan van Rossum [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8673-2514>),
  Fred van Eeuwijk [ctb] (ORCID: <https://orcid.org/0000-0003-3672-2921>)
Maintainer: Bart-Jan van Rossum <bart-jan.vanrossum@wur.nl>
Repository: CRAN
Date/Publication: 2026-05-18 13:20:02 UTC
Built: R 4.5.2; ; 2026-05-18 14:26:33 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,GWAS)
S3method(plot,QTLMPP)
S3method(plot,gDataMPP)
S3method(print,summary.QTLMPP)
S3method(print,summary.gDataMPP)
S3method(summary,QTLMPP)
S3method(summary,gDataMPP)
export(calcIBDMPP)
export(createGDataMPP)
export(kinshipIBD)
export(readRABBITMPP)
export(selQTLMPP)
import(grDevices)
import(graphics)
import(statgenGWAS)
importFrom(foreach,`%do%`)
importFrom(foreach,`%dopar%`)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,model.matrix)
importFrom(stats,pchisq)
importFrom(stats,setNames)
importFrom(utils,getFromNamespace)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
importFrom(utils,tail)
importFrom(utils,unzip)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# statgenMPP <img src="man/figures/logo.png" align="right" height="139" alt="" />

[![](https://www.r-pkg.org/badges/version/statgenMPP)](https://www.r-pkg.org/pkg/statgenMPP)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/statgenMPP)](https://www.r-pkg.org/pkg/statgenMPP)
[![R-CMD-check](https://github.com/Biometris/statgenMPP/workflows/R-CMD-check/badge.svg)](https://github.com/Biometris/statgenMPP/actions?workflow=R-CMD-check)
[![codecov](https://codecov.io/gh/Biometris/statgenMPP/branch/main/graph/badge.svg)](https://app.codecov.io/gh/Biometris/statgenMPP)

The statgenMPP package is developed as an easy-to-use package for QTL
mapping in biparental and multi-parent populations. The package has many
ways of visualizing inputs and results. First Identity By Descent (IBD)
probabilities are computed using Hidden Markov Models. These
probabilities are then used in a mixed model approach for QTL Mapping as
described in [Li et
al. 2021](https://link.springer.com/article/10.1007/s00122-021-03919-7).

- Install from CRAN:

``` r
install.packages("statgenMPP")
```

- Install latest development version from GitHub (requires
  [remotes](https://github.com/r-lib/remotes) package):

``` r
remotes::install_github("Biometris/statgenMPP", ref = "develop", dependencies = TRUE)
```

## Example

Here we give a simple example, using a single biparental population. The
example contains simulated data for one F4DH population. The population
type F4DH is a cross between two parents, A and C, followed by 3
generations of selfings, followed by a DH generation, see
[statgenIBD](https://biometris.github.io/statgenIBD/) for details.

First we load the marker data and phenotypic data and calculate the IBDs
using the `calcIBDMPP` function:

``` r
library(statgenMPP)
#> Loading required package: statgenGWAS

markerFiles <- system.file("extdata/multipop", "AxC.txt", 
                             package = "statgenMPP")
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")
phenoDat <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                   package = "statgenMPP"))

ACMPP <- calcIBDMPP(crossNames = c("AxC"),  
                     markerFiles = markerFiles,
                     pheno = phenoDat,
                     popType = "F4DH",
                     mapFile = mapFile,
                     evalDist = 5)
```

The population has the following simple structure, for more complicated
examples see the vignette.

``` r
plot(ACMPP, plotType = "pedigree")
```

<img src="man/figures/README-plotPACMPP-1.png" width="100%" />

The next step is to select QTLs using `selQTLMPP`:

``` r
ACMQM <- selQTLMPP(MPPobj = ACMPP, trait = "yield", threshold = 3)
```

The QTL-mapping profile and parental effects:

``` r
plot(ACMQM, plotType = "QTLProfileExt")
```

<img src="man/figures/README-plotQPEACMQM-1.png" width="100%" />

The confidence intervals around the parental effects for each QTL:

``` r
plot(ACMQM, plotType = "parCIs")
```

<img src="man/figures/README-plotCIACMQM-1.png" width="100%" />

A summary of the QTL-analyis gives a short overview containing the total
number of markers and the number of QTLs found. Also for all QTLs their
position on the chromosome is shown as well as the nearest marker on the
original map, the explained variance, and the effects and the standard
errors of all parents:

``` r
## Print summary
summary(ACMQM)
#> Trait analysed: yield 
#> 
#> Data are available for 95 markers.
#> Threshold: 3 
#> 
#> Number of QTLs: 2 
#> 
#>   evalPos chr pos mrkNear minlog10p varExpl  eff_A  eff_C se_eff_A se_eff_C
#>  EXT_1_20   1  20    M1_3      9.25   0.280 -0.921  0.921    0.125    0.125
#>  EXT_3_65   3  65    M3_7     12.50   0.513  1.251 -1.251    0.138    0.138
```

<!-- README.md is generated from README.Rmd. Please edit that file -->

# statgenMPP <img src="man/figures/logo.png" align="right" height="139" alt="" />

[![](https://www.r-pkg.org/badges/version/statgenMPP)](https://www.r-pkg.org/pkg/statgenMPP)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/statgenMPP)](https://www.r-pkg.org/pkg/statgenMPP)
[![R-CMD-check](https://github.com/Biometris/statgenMPP/workflows/R-CMD-check/badge.svg)](https://github.com/Biometris/statgenMPP/actions?workflow=R-CMD-check)
[![codecov](https://codecov.io/gh/Biometris/statgenMPP/branch/main/graph/badge.svg)](https://app.codecov.io/gh/Biometris/statgenMPP)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.14849990.svg)](https://doi.org/10.5281/zenodo.14849990)

The statgenMPP package is developed as an easy-to-use package for QTL
mapping in biparental and multi-parent populations. The package has many
ways of visualizing inputs and results. First Identity By Descent (IBD)
probabilities are computed using Hidden Markov Models. These
probabilities are then used in a mixed model approach for QTL Mapping as
described in [Li et
al. 2021](https://link.springer.com/article/10.1007/s00122-021-03919-7).

- Install from CRAN:

``` r
install.packages("statgenMPP")
```

- Install latest development version from GitHub (requires
  [remotes](https://github.com/r-lib/remotes) package):

``` r
remotes::install_github("Biometris/statgenMPP", ref = "develop", dependencies = TRUE)
```

## Example

Here we give a simple example, using a single biparental population. The
example contains simulated data for one F4DH population. The population
type F4DH is a cross between two parents, A and C, followed by 3
generations of selfings, followed by a DH generation, see
[statgenIBD](https://biometris.github.io/statgenIBD/) for details.

First we load the marker data and phenotypic data and calculate the IBDs
using the `calcIBDMPP` function:

``` r
library(statgenMPP)
#> Loading required package: statgenGWAS

markerFiles <- system.file("extdata/multipop", "AxC.txt", 
                             package = "statgenMPP")
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")
phenoDat <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                   package = "statgenMPP"))

ACMPP <- calcIBDMPP(crossNames = c("AxC"),  
                     markerFiles = markerFiles,
                     pheno = phenoDat,
                     popType = "F4DH",
                     mapFile = mapFile,
                     evalDist = 5)
```

The population has the following simple structure, for more complicated
examples see the vignette.

``` r
plot(ACMPP, plotType = "pedigree")
```

<img src="man/figures/README-plotPACMPP-1.png" width="100%" />

The next step is to select QTLs using `selQTLMPP`:

``` r
ACMQM <- selQTLMPP(MPPobj = ACMPP, trait = "yield", threshold = 3)
```

The QTL-mapping profile and parental effects:

``` r
plot(ACMQM, plotType = "QTLProfileExt")
```

<img src="man/figures/README-plotQPEACMQM-1.png" width="100%" />

The confidence intervals around the parental effects for each QTL:

``` r
plot(ACMQM, plotType = "parCIs")
```

<img src="man/figures/README-plotCIACMQM-1.png" width="100%" />

A summary of the QTL-analyis gives a short overview containing the total
number of markers and the number of QTLs found. Also for all QTLs their
position on the chromosome is shown as well as the nearest marker on the
original map, the explained variance, and the effects and the standard
errors of all parents:

``` r
## Print summary
summary(ACMQM)
#> Trait analysed: yield 
#> 
#> Data are available for 95 markers.
#> Threshold: 3 
#> 
#> Number of QTLs: 2 
#> 
#>   evalPos chr pos mrkNear minlog10p varExpl  eff_A  eff_C se_eff_A se_eff_C
#>  EXT_1_20   1  20    M1_3      9.25   0.280 -0.921  0.921    0.125    0.125
#>  EXT_3_65   3  65    M3_7     12.50   0.513  1.251 -1.251    0.138    0.138
```

## ----setup, include = FALSE-------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.dim = c(7, 4)
)
library(statgenMPP)
op <- options(width = 90, 
              digits = 3)

## ----simMarkerDat, results='asis', echo=FALSE-------------------------------------------
simMrkDat <- read.delim(system.file("extdata/multipop", "AxB.txt", package = "statgenMPP"))
knitr::kable(simMrkDat[1:4, 1:5])

## ----simPhenoDat, results='asis', echo=FALSE--------------------------------------------
simPhenoDat <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt", package = "statgenMPP"))
knitr::kable(head(simPhenoDat))

## ----simIBD-----------------------------------------------------------------------------
## Specify files containing markers.
# One file for each of the two crosses.
markerFiles <- c(system.file("extdata/multipop", "AxB.txt", 
                             package = "statgenMPP"),
                 system.file("extdata/multipop", "AxC.txt", 
                             package = "statgenMPP"))

## Specify file containing map.
# Both crosses use the same map file.
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")

## Read phenotypic data
phenoDat <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                   package = "statgenMPP"))
# Check contents.
head(phenoDat)

## Perform IBD calculations. 
ABCMPP <- calcIBDMPP(crossNames = c("AxB", "AxC"),  
                     markerFiles = markerFiles,
                     pheno = phenoDat,
                     popType = "F4DH",
                     mapFile = mapFile,
                     evalDist = 5)

## ----sumABCMPP--------------------------------------------------------------------------
## Print summary
summary(ABCMPP)

## ----plotPABCMPP------------------------------------------------------------------------
## Plot structure of the pedigree.
plot(ABCMPP, plotType = "pedigree")

## ----plotGABCMPP------------------------------------------------------------------------
## Plot genetic map.
# Highlight marker on chromosome 3 at position 40.
plot(ABCMPP, plotType = "genMap", highlight = "EXT_3_40")

## ----plotSGABCMPP, fig.height=10--------------------------------------------------------
## Plot IBD probabilities for genotype AxB0001.
plot(ABCMPP, plotType = "singleGeno", genotype = "AxB0001")

## ----ABCSQM-----------------------------------------------------------------------------
## Perform Single-QTL Mapping.
ABCSQM <- selQTLMPP(MPPobj = ABCMPP,
                    trait = "yield",
                    maxCofactors = 0)

## ----QPABCSQM---------------------------------------------------------------------------
## Plot QTL Profile for ABC SQM.
plot(ABCSQM, plotType = "QTLProfile")

## ----ABCMQM-----------------------------------------------------------------------------
## Perform Multi-QTL Mapping.
ABCMQM <- selQTLMPP(MPPobj = ABCMPP,
                    trait = "yield",
                    threshold = 3)

## ----ABCMQM_kin, eval=FALSE-------------------------------------------------------------
# ## Perform Multi-QTL Mapping.
# # Compute kinship matrices.
# ABCMQM_kin <- selQTLMPP(MPPobj = ABCMPP,
#                         trait = "yield",
#                         threshold = 3,
#                         computeKin = TRUE)

## ----plotQRABCMQM-----------------------------------------------------------------------
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "QTLRegion")

## ----plotQPABCMQM-----------------------------------------------------------------------
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "QTLProfile")

## ----plotPEABCMQM-----------------------------------------------------------------------
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "parEffs")

## ----plotQPEABCMQM----------------------------------------------------------------------
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "QTLProfileExt")

## ----plotCIABCMQM-----------------------------------------------------------------------
## Plot confidence intervals for parental effects for ABC MQM.
plot(ABCMQM, plotType = "parCIs")

## ----sumABCMQM--------------------------------------------------------------------------
## Print summary
summary(ABCMQM)

## ----extractABCRes----------------------------------------------------------------------
## Extract results of QTL mapping.
ABCMQMres <- ABCMQM$GWAResult$yield
head(ABCMQMres[, 1:8])

## ----extractABCQTL----------------------------------------------------------------------
## Extract QTLs and markers within QTL windows.
ABCMQMQTL <- ABCMQM$signSnp$yield
head(ABCMQMQTL[, c(2:8, 10)])

## ----maizeIBD---------------------------------------------------------------------------
## Define names of crosses.
crosses <- paste0("F353x", c("B73", "D06", "D09", "EC169", "F252", "F618", 
                             "Mo17", "UH250", "UH304", "W117"))
head(crosses)

## Specify files containing crosses.
## Extract them in a temporary directory.
tempDir <- tempdir()
crossFiles <- unzip(system.file("extdata/maize/maize.zip", package = "statgenMPP"), 
                    files = paste0(crosses, ".txt"), exdir = tempDir)

## Specify file containing map.
mapFile <- unzip(system.file("extdata/maize/maize.zip", package = "statgenMPP"), 
                 files = "map.txt", exdir = tempDir)

## Read phenotypic data.
phenoFile <- unzip(system.file("extdata/maize/maize.zip", package = "statgenMPP"),
                   files = "EUmaizePheno.txt", exdir = tempDir)
phenoDat <- read.delim(phenoFile)
head(phenoDat[, 1:5])

## Perform IBD calculations. 
maizeMPP <- calcIBDMPP(crossNames = crosses, 
                       markerFiles = crossFiles,
                       pheno = phenoDat,
                       popType = "DH",
                       mapFile = mapFile,
                       evalDist = 5)

## ----sumMaizeIBD------------------------------------------------------------------------
## Print summary
summary(maizeMPP)

## ----plotPmaizeIBD----------------------------------------------------------------------
## Plot structure of the pedigree.
plot(maizeMPP, plotType = "pedigree")

## ----maizeSQM, eval=FALSE---------------------------------------------------------------
# ## Perform Single-QTL Mapping.
# maizeSQM <- selQTLMPP(MPPobj = maizeMPP,
#                       trait = "mean_DtSILK",
#                       maxCofactors = 0)

## ----QPmaizeSQM-------------------------------------------------------------------------
## Plot QTL Profile for maize SQM.
plot(maizeSQM, plotType = "QTLProfile")

## ----maizeMQM, eval=FALSE---------------------------------------------------------------
# ## Perform Multi-QTL Mapping.
# maizeMQM <- selQTLMPP(MPPobj = maizeMPP,
#                       trait = "mean_DtSILK",
#                       threshold = 5)

## ----plotQPEmaizeMQM--------------------------------------------------------------------
## Plot QTL Profile for maize MQM.
plot(maizeMQM, plotType = "QTLProfileExt")

## ----plotCIamizeMQM---------------------------------------------------------------------
## Plot confidence intervals for parental effects for maize MQM.
plot(maizeMQM, plotType = "parCIs")

## ----barleyIBD--------------------------------------------------------------------------
## Specify files containing RABBIT output.
## Extract in a temporary directory.
tempDir <- tempdir()
inFile <- unzip(system.file("extdata/barley/barley_magicReconstruct.zip", 
                            package = "statgenMPP"), exdir = tempDir)

## Specify pedigree file.
pedFile <- system.file("extdata/barley/barley_pedInfo.csv",
                       package = "statgenMPP")

## Read phenotypic data.
data("barleyPheno")

## read RABBIT output. 
barleyMPP <- readRABBITMPP(infile = inFile,
                           pedFile = pedFile,
                           pheno = barleyPheno)

## ----sumPbarleyIBD----------------------------------------------------------------------
## Summary.
summary(barleyMPP)

## ----barleyMQM, eval=FALSE--------------------------------------------------------------
# ## Perform Multi-QTL Mapping with threshold 4.
# barleyMQM <- selQTLMPP(MPPobj = barleyMPP,
#                        trait = "Awn_length",
#                        threshold = 4)

## ----QPbarleyMQM16----------------------------------------------------------------------
## Plot QTL Profile for barley MQM - chromosome 1-6.
plot(barleyMQM, plotType = "QTLProfileExt", chr = 1:6)

## ----QPbarleyMQM7-----------------------------------------------------------------------
## Plot QTL Profile for barley MQM - chromosome 7.
plot(barleyMQM, plotType = "QTLProfileExt", chr = 7)

## ----plotCIbarleyMQM--------------------------------------------------------------------
## Plot confidence intervals for parental effects for maize MQM.
plot(barleyMQM, plotType = "parCIs")

## ----sumBarleyMQM-----------------------------------------------------------------------
## Summary.
summary(barleyMQM)

## ----ABCMQMPar, eval = FALSE------------------------------------------------------------
# ## Register parallel back-end with 2 cores.
# doParallel::registerDoParallel(cores = 2)
# 
# ## Perform Multi-QTL Mapping.
# ABCMQM_Par <- selQTLMPP(MPPobj = ABCMPP,
#                         trait = "yield",
#                         threshold = 3,
#                         parallel = TRUE)

## ----winddown, include = FALSE------------------------------------------------
options(op)

#' one-round genome scan with specified cofactor(s)
#'
#' @importFrom stats coef pchisq model.matrix
#' @keywords internal
scanQTL <- function(modDat,
                    map,
                    markers,
                    parents,
                    QTLwindow = 10,
                    cof = NULL,
                    Usc = NULL,
                    trait = NULL,
                    maxIter = 100,
                    se = FALSE,
                    parallel = FALSE,
                    verbose = FALSE) {
  ## Get info from input.
  nPar <- length(parents)
  nGeno <- nrow(modDat)
  nCross <- length(unique(modDat[["cross"]]))
  ## Create general model matrices.
  y <- modDat[[trait]]
  X <- spam::as.spam(model.matrix(if (nCross > 1) ~cross else ~ 1,
                                  data = modDat))
  lRinv <- constructRinv(modDat, residual = ~cross, weights = 1)
  ## Fit NULL model with all cofactors.
  Z0 <- do.call(spam::cbind.spam, lapply(X = cof, FUN = function(mrk) {
    markers[, mrk, ]
  }))
  ## Fit models for each marker.
  chrs <- unique(map[["chr"]])
  `%op%` <- getOper(parallel && foreach::getDoParRegistered())
  scanFull <- foreach::foreach(i = seq_along(chrs)) %op% {
    UscChr <- Usc[[i]]
    nEigChr <- ncol(UscChr)
    ## Add genotype.
    lGinv0 <- lapply(X = seq_along(cof), FUN = function(i) {
      spam::diag.spam(x = c(rep(0, nPar * (i - 1)),
                            rep(1, nPar),
                            rep(0, nPar * (length(cof) - i) +
                                  if (!is.null(Usc)) nEigChr else 0)))
    })
    names(lGinv0) <- cof
    if (!is.null(UscChr)) {
      ZChr <- spam::cbind.spam(Z0, UscChr)
      lGinv1 <- spam::diag.spam(x = c(rep(0, times = length(cof) * nPar),
                                      rep(1, nEigChr)))
      LGinvChr <- c(lGinv0, list(K = lGinv1))
    } else {
      ZChr <- Z0
      LGinvChr <- lGinv0
    }
    ## Fit NULL model with all cofactors.
    fitModNULL <- sparseMixedModels(y = y, X = X, Z = ZChr,
                                    lRinv = lRinv, lGinv = LGinvChr,
                                    tolerance = 1e-3)
    chrMrk <- rownames(map)[map[["chr"]] == chrs[i]]
    QTLRegion <- setNames(logical(length = length(chrMrk)), chrMrk)
    minlog10p <- setNames(numeric(length = length(chrMrk)), chrMrk)
    effects <- seEffects <- matrix(nrow = length(chrMrk), ncol = nPar,
                                   dimnames = list(chrMrk, parents))
    for (scanMrk in chrMrk) {
      ## Get cofactors for current markers.
      cofMrk <- selectCofactors(map = map,
                                marker = scanMrk,
                                cofactors = cof,
                                QTLwindow = QTLwindow)
      selMrk <- c(scanMrk, cofMrk)
      ZMrk <- do.call(spam::cbind.spam, lapply(X = selMrk, FUN = function(mrk) {
        markers[, mrk, ]
      }))
      lGinvMrk <- lapply(X = seq_along(selMrk), FUN = function(i) {
        spam::diag.spam(x = c(rep(0, nPar * (i - 1)),
                              rep(1, nPar),
                              rep(0, nPar * (length(selMrk) - i) +
                                    if (!is.null(UscChr)) nEigChr else 0)))
      })
      names(lGinvMrk) <- selMrk
      if (!is.null(UscChr)) {
        ZMrk <- spam::cbind.spam(ZMrk, UscChr)
        lGinv1Mrk <- spam::diag.spam(x = c(rep(0, times = length(selMrk) * nPar),
                                           rep(1, nEigChr)))
        lGinvMrk <- c(lGinvMrk, list(K = lGinv1Mrk))
      }
      ## Fit model for current marker.
      fitModMrk <- sparseMixedModels(y = y, X = X, Z = ZMrk,
                                     lRinv = lRinv, lGinv = lGinvMrk,
                                     tolerance = 1e-3)
      ## Compute change in deviance.
      dev <- 2 * fitModMrk$logL - 2 * fitModNULL$logL
      ## Refit NULL model only if cofactors differ for current marker.
      if (length(cofMrk) != length(cof)) {
        ## Fit NULL model with restricted cofactors.
        ZMrk2 <- do.call(spam::cbind.spam, lapply(X = cofMrk,
                                                  FUN = function(mrk) {
                                                    markers[, mrk, ]
                                                  }))
        lGinvMrk2 <- lapply(X = seq_along(cofMrk), FUN = function(i) {
          spam::diag.spam(x = c(rep(0, nPar * (i - 1)),
                                rep(1, nPar),
                                rep(0, nPar * (length(cofMrk) - i) +
                                      if (!is.null(UscChr)) nEigChr else 0)))
        })
        names(lGinvMrk2) <- cofMrk
        if (!is.null(UscChr)) {
          ZMrk2 <- cbind(ZMrk2, UscChr)
          lGinv1Mrk2 <- spam::diag.spam(x = c(rep(0, times = length(cofMrk) * nPar),
                                              rep(1, nEigChr)))
          lGinvMrk2 <- c(lGinvMrk2, list(K = lGinv1Mrk2))
        }
        fitModCof <- sparseMixedModels(y = y, X = X, Z = ZMrk2,
                                       lRinv = lRinv, lGinv = lGinvMrk2,
                                       tolerance = 1e-3)
        dev <- 2 * fitModMrk$logL - 2 * fitModCof$logL
      }
      QTLRegion[scanMrk] <- length(cofMrk) != length(cof)
      minlog10p[scanMrk] <- min(-log10(0.5 * pchisq(dev, 1,
                                                    lower.tail = FALSE)), 300)
      effects[scanMrk, ] <- fitModMrk$a[(1 + nCross):(length(parents) + nCross)]
      if (se) {
        Cinv <- spam::solve.spam(fitModMrk$C)
        Dg <- spam::spam(x = 0, nrow = nPar, ncol = nrow(fitModMrk$C))
        I <- spam::diag.spam(1, nPar)
        J <- spam::spam(x = 1 / nPar, nrow = nPar, ncol = nPar)
        Dg[1:nPar, (nCross + 1):(nCross + nPar)] <- I - J
        seEffects[scanMrk, ] <-
          as.vector(sqrt(spam::diag.spam(Dg %*% Cinv %*% t(Dg))))
      }
    }
    list(QTLRegion, minlog10p, effects, seEffects)
  }
  QTLRegion <- do.call(c, lapply(X = scanFull, FUN = `[[`, 1))
  minlog10p <- do.call(c, lapply(X = scanFull, FUN = `[[`, 2))
  effects <- do.call(rbind, args = lapply(X = scanFull, FUN = `[[`, 3))
  seEffects <- do.call(rbind, args = lapply(X = scanFull, FUN = `[[`, 4))
  dimnames(effects) <- list(rownames(map), paste0("eff_", parents))
  dimnames(seEffects) <- list(rownames(map), paste0("se_eff_", parents))
  scanRes <- data.table::data.table(trait = trait,
                                    snp = rownames(map),
                                    map,
                                    pValue = 10 ^ -minlog10p,
                                    effects,
                                    seEffects,
                                    minlog10p = minlog10p,
                                    QTLRegion = QTLRegion)
  return(scanRes)
}

#' Summary function for the class \code{QTLMPP}
#'
#' Gives a summary for an object of S3 class \code{QTLMPP}.
#'
#' @param object An object of class \code{QTLMPP}.
#' @param ... Not used.
#'
#' @examples
#' \dontrun{
#' ## Read phenotypic data.
#' pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
#'                                package = "statgenMPP"))
#' ## Rename first column to genotype.
#' colnames(pheno)[1] <- "genotype"
#'
#' ## Compute IBD probabilities for simulated population - AxB, AxC.
#' ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
#'                   markerFiles = c(system.file("extdata/multipop", "AxB.txt",
#'                                               package = "statgenMPP"),
#'                                   system.file("extdata/multipop", "AxC.txt",
#'                                               package = "statgenMPP")),
#'                   pheno = pheno,
#'                   popType = "F4DH",
#'                   mapFile = system.file("extdata/multipop", "mapfile.txt",
#'                                         package = "statgenMPP"),
#'                   evalDist = 5)
#'
#' ## Multi-QTL Mapping.
#' ABC_MQM <- selQTLMPP(ABC, trait = "yield")
#'
#' ## Print summary.
#' summary(ABC_MQM)
#' }
#'
#' @export
summary.QTLMPP <- function(object,
                           ...) {
  GWAResult <- object$GWAResult[[1]]
  signSnp <- object$signSnp[[1]]
  GWASInfo <- object$GWASInfo
  trait <- unique(GWAResult$trait)
  parents <- GWASInfo$parents
  ## Restrict signSnp to QTLs found.
  signSnp <- signSnp[signSnp[["snpStatus"]] == "significant SNP", ]
  ## Rename columns.
  signSnp[["minlog10p"]] <- signSnp[["LOD"]]
  signSnp[["evalPos"]] <- signSnp[["snp"]]
  ## Restrict columns for nicer output.
  signSnp <- signSnp[, c("evalPos", "chr", "pos", "mrkNear", "minlog10p",
                         "varExpl", paste0("eff_", parents),
                         paste0("se_eff_", parents)), with = FALSE]
  ## Add attributes used for printing.
  res <- structure(signSnp,
                   class = c("summary.QTLMPP", "data.frame"),
                   trait = unique(GWAResult$trait),
                   nMrk = length(unique(GWAResult$snp)),
                   thr = object$thr[[1]][trait])
  return(res)
}

#' Print summary of object of class \code{summary.QTLMPP}
#'
#' @param x An object of class \code{summary.QTLMPP}.
#' @param ... Not used.
#'
#' @export
print.summary.QTLMPP <- function(x,
                                 ...) {
  ## Print traits.
  cat("Trait analysed:", attr(x, which = "trait"), "\n\n")
  ## Print SNP numbers.
  cat("Data are available for", attr(x, which = "nMrk"), "markers.\n")
  ## Print significant SNP info.
  cat("Threshold:", attr(x, which = "thr"), "\n\n")
  if (nrow(x) > 0) {
    nSignSnp <- nrow(x)
    cat("Number of QTLs:", nSignSnp, "\n\n")
    print(data.table::data.table(x), row.names = FALSE, digits = 3,
          class = FALSE)
    cat("\n")
  } else {
    cat("No QTLs found.", "\n\n")
  }
}

#' Plot function for the class \code{QTLMPP}
#'
#' Creates a plot of an object of S3 class \code{QTLMPP}. The following types of
#' plot can be made:
#' \describe{
#' \item{QTLProfile}{A QTL profile plot, i.e. a plot of \eqn{-log10(p)} values
#' per marker.}
#' \item{parEffs}{A plot of effect sizes and directions per parent.}
#' \item{QTLRegion}{A plot highlighting the QTLs found on the genetic map.}
#' \item{QTLProfileExt}{A combination of the QTL profile and parental effects
#' plotted above each other.}
#' \item{parCIs}{A plot of the effect sizes and confidence intervals for each
#' parent for each of the QTLs found.}
#' }
#' See details for a detailed description of the plots and the plot options
#' specific to the different plots.
#'
#' @section QTL Profile plot:
#' A profile of all marker positions and corresponding \eqn{-log10(p)} values is
#' plotted. QTLs found are highlighted with red dots. The threshold is plotted
#' as a horizontal line. If there are previously known marker effects, false
#' positives and true negatives can also be marked.\cr
#' Extra parameter options:
#' \describe{
#' \item{\code{xLab}}{A character string, the x-axis label. Default =
#' \code{"Chromosomes"}}
#' \item{\code{yLab}}{A character string, the y-axis label. Default =
#' \code{-log10(p)}}
#' \item{\code{effects}}{A character vector, indicating which markers correspond
#' to a real (known) effect. Used for determining true/false positives and
#' false negatives. True positives are colored green, false positives orange and
#' false negatives yellow.}
#' \item{\code{colPalette}}{A color palette used for plotting. Default
#' coloring is done by chromosome, using black and grey.}
#' \item{\code{signLwd}}{A numerical value giving the thickness of the
#' points that are false/true positives/negatives. Default = 0.6}
#' \item{\code{chr}}{A vector of chromosomes to be plotted. By default, all
#' chromosomes are plotted. Using this option allows restricting the plot to a
#' subset of chromosomes.}
#' }
#'
#' @section Parental effects Plot:
#' A plot of effect sizes for each of the parents for the QTLs found is
#' created.\cr
#' Extra parameter options:
#' \describe{
#' \item{\code{xLab}}{A character string, the x-axis label. Default =
#' \code{"Chromosomes"}}
#' \item{\code{yLab}}{A character string, the y-axis label. Default =
#' \code{"Parents"}}
#' \item{\code{chr}}{A vector of chromosomes to be plotted. By default, all
#' chromosomes are plotted. Using this option allows restricting the plot to a
#' subset of chromosomes.}
#' }
#'
#' @section QTL Region Plot:
#' A plot of effect is created on which the positions of the QTLs found in
#' the QTL detection are highlighted in red.\cr
#' No extra parameter options.
#'
#' @section Extended QTL Profile Plot:
#' An extended version of the QTL Profile Plot, in which the QLT profile plot is
#' combined with the parental effect plot to make it easier to assess the
#' effects for each specific QTL found.\cr
#' Extra parameter options:
#' \describe{
#' \item{\code{chr}}{A vector of chromosomes to be plotted. By default, all
#' chromosomes are plotted. Using this option allows restricting the plot to a
#' subset of chromosomes.}
#' }
#'
#' @section Parental Confidence Interval Plot:
#' A plot of the parental effects and the confidence intervals around them for
#' each of the QTLs found. Positive significant effects are shown in red,
#' negative significant effects in blue, and all other effects are shown in
#' black.
#'
#' @param x An object of class \code{QTLMPP}.
#' @param ... further arguments to be passed on to the actual plotting
#' functions.
#' @param plotType A character string indicating the type of plot to be made.
#' One of "QTLProfile", "parEffs", and "QTLRegion".
#' @param title A character string, the title of the plot.
#' @param output Should the plot be output to the current device? If
#' \code{FALSE}, only a ggplot object is invisibly returned.
#'
#' @examples
#' \dontrun{
#' ## Read phenotypic data.
#' pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
#'                                package = "statgenMPP"))
#' ## Rename first column to genotype.
#' colnames(pheno)[1] <- "genotype"
#'
#'
#' ## Compute IBD probabilities for simulated population - AxB, AxC.
#' ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
#'                   markerFiles = c(system.file("extdata/multipop", "AxB.txt",
#'                                               package = "statgenMPP"),
#'                                   system.file("extdata/multipop", "AxC.txt",
#'                                               package = "statgenMPP")),
#'                   pheno = pheno,
#'                   popType = "F4DH",
#'                   mapFile = system.file("extdata/multipop", "mapfile.txt",
#'                                         package = "statgenMPP"),
#'                   evalDist = 5)
#'
#' ## Multi-QTL Mapping.
#' ABC_MQM <- selQTLMPP(ABC, trait = "yield")
#'
#' ## QTL Profile plot.
#' plot(ABC_MQM, plotType = "QTLProfile")
#'
#' ## Plot of parental effects for QTLs found.
#' plot(ABC_MQM, plotType = "parEffs")
#'
#' ## Plot of genetic map highlighting positions of QTLs found.
#' plot(ABC_MQM, plotType = "QTLRegion")
#'
#' ## Extended QTL Profile plot.
#' plot(ABC_MQM, plotType = "QTLProfileExt")
#'
#' ## Parental confidence interval plot.
#' plot(ABC_MQM, plotType = "parCIs")
#' }
#'
#' @export
plot.QTLMPP <- function(x,
                        ...,
                        plotType = c("QTLProfile", "parEffs", "QTLRegion",
                                     "QTLProfileExt", "parCIs"),
                        title = NULL,
                        output = TRUE) {
  plotType <- match.arg(plotType)
  if (plotType == "QTLProfile") {
    p <- plot.GWAS(x = x, ... = ..., plotType = "manhattan", trial = NULL,
                   trait = NULL, title = title, type = "lines", output = output)
  } else {
    dotArgs <- list(...)
    ## Get results.
    GWAResult <- x$GWAResult[[1]]
    ## Get peaks.
    signSnp <- x$signSnp[[1]]
    if (plotType != "QTLRegion" && !is.null(dotArgs$chr)) {
      GWAResult <- droplevels(GWAResult[GWAResult$chr %in% dotArgs$chr, ])
      if (nrow(GWAResult) == 0) {
        stop("Select at least one valid chromosome for plotting.\n")
      }
      signSnp <- droplevels(signSnp[signSnp$chr %in% dotArgs$chr, ])
    }
    ## Compute chromosome boundaries and map.
    GWAResComp <- GWAResult[GWAResult[["trait"]] == GWAResult[["trait"]][1], ]
    GWAResComp[["chr"]] <- factor(GWAResComp[["chr"]],
                                  levels = unique(GWAResComp[["chr"]]))
    chrBnd <- aggregate(x = GWAResComp$pos, by = list(GWAResComp$chr),
                        FUN = max)
    ## Compute cumulative positions.
    addPos <- data.frame(chr = chrBnd[, 1],
                         add = c(0, cumsum(chrBnd[, 2]))[1:nrow(chrBnd)],
                         stringsAsFactors = FALSE)
    map <- GWAResComp[, c("snp", "chr", "pos", "LOD")]
    map <- merge(map, addPos, by = "chr")
    map[["cumPos"]] <- map[["pos"]] + map[["add"]]
    if (plotType == "parEffs") {
      parents <- x$GWASInfo$parents
      trait <- GWAResult[["trait"]][1]
      ## Create effect data - basically a long format version of GWAResult
      effectDat <- lapply(X = parents, FUN = function(parent) {
        parDat <- GWAResult
        parDat[["trait"]] <- parent
        parDat[["effect"]] <- parDat[[paste0("eff_", parent)]]
        return(parDat)
      })
      effectDat <- do.call(rbind, effectDat)
      effectDat[["trait"]] <- factor(effectDat[["trait"]],
                                     levels = parents)
      ## Convert signSnp to long format as well.
      if (nrow(signSnp) > 0) {
        signSnpLong <- lapply(X = parents, FUN = function(parent) {
          parDat <- signSnp
          parDat[["trait"]] <- parent
          parDat[["effect"]] <- parDat[[paste0("eff_", parent)]]
          return(parDat)
        })
        signSnpLong <- do.call(rbind, signSnpLong)
      } else {
        signSnpLong <- NULL
      }
      p <- do.call(effectPlot,
                   args = c(list(effectDat = effectDat,
                                 signSnp = signSnpLong,
                                 map = map,
                                 chrBoundaries = chrBnd,
                                 title = title,
                                 trait = trait,
                                 output = output),
                            dotArgs[!(names(dotArgs) %in% c("effectDat", "signSnp",
                                                            "map", "chrBoundaries"))]))

    } else if (plotType == "QTLRegion") {
      ## Restrict signSnp to QTLs.
      signSnp <- signSnp[signSnp[["snpStatus"]] == "significant SNP", ]
      p <- do.call(geneticMapPlot,
                   args = c(list(map = map,
                                 highlight = if (nrow(signSnp) > 0) signSnp,
                                 title = title,
                                 output = output),
                            dotArgs[!(names(dotArgs) %in% c("highlight", "map"))]))
    } else if (plotType == "QTLProfileExt") {
      ## Construct title.
      if (is.null(title)) {
        title <- paste("QTL Profile (upper panel) and parental effects",
                       "(lower panel):", GWAResult[["trait"]][1])
      }
      ## Construct data for vertical lines in QTL profile.
      ## Center line in empty space between chromosomes.
      vertDat <- addPos
      minPos <- aggregate(x = GWAResComp$pos, by = list(GWAResComp$chr),
                          FUN = min)
      vertDat[["x"]] <- vertDat[["add"]] + c(0, minPos[-1, "x"]) / 2
      ## Add title here to assure font is the same as for other plots.
      p1 <- plot(x, chr = dotArgs$chr, plotType = "QTLProfile", title = title,
                 output = FALSE) +
        ggplot2::geom_vline(ggplot2::aes(xintercept = .data[["x"]]),
                            linetype = "dashed", data = vertDat[-1, ]) +
        ggplot2::theme(axis.ticks.x = ggplot2::element_blank(),
                       axis.text.x = ggplot2::element_blank(),
                       axis.title.x = ggplot2::element_blank(),
                       plot.margin = ggplot2::unit(c(0.2, 0, 0, 0), "cm"),
                       panel.background = ggplot2::element_blank(),
                       panel.border = ggplot2::element_rect(fill = NA,
                                                            color = "black",
                                                            linewidth = 0.5,
                                                            linetype = "solid"))
      p2 <- plot(x, chr = dotArgs$chr, plotType = "parEffs", title = "",
                 output = FALSE) +
        ggplot2::theme(plot.margin = ggplot2::unit(c(0, 0, 0, 0), "cm"))
      ## Get widths.
      g1 <- ggplot2::ggplotGrob(p1)
      g2 <- ggplot2::ggplotGrob(p2)
      g1Widths <- g1$widths[-1]
      g2Widths <- g2$widths[-1]
      maxWidths <- grid::unit.pmax(g1Widths, g2Widths)
      ## Set widths.
      g1$widths[-1] <- maxWidths
      g2$widths[-1] <- maxWidths
      p <- gridExtra::grid.arrange(g1, g2)
    } else if (plotType == "parCIs") {
      parents <- x$GWASInfo$parents
      trait <- GWAResult[["trait"]][1]
      signSnp <- signSnp[signSnp[["snpStatus"]] == "significant SNP", ]
      ## Create QTL effect data - basically a long format version of signSnp
      parCIsDat <- lapply(X = parents, FUN = function(parent) {
        parDat <- signSnp
        parDat[["snp"]] <- paste("QTL at chr", parDat[["chr"]],
                                 round(parDat[["pos"]], 0), "cM")
        parDat[["parent"]] <- parent
        parDat[["effect"]] <- parDat[[paste0("eff_", parent)]]
        parDat[["seEffect"]] <- parDat[[paste0("se_eff_", parent)]]
        return(parDat)
      })
      parCIsDat <- do.call(rbind, parCIsDat)
      parCIsDat <- parCIsDat[c("snp", "parent", "effect", "seEffect")]
      parCIsDat[["parent"]] <- factor(parCIsDat[["parent"]],
                                         levels = parents)
      p <- do.call(parCIsPlot,
                   args = c(list(parCIsDat = parCIsDat,
                                 title = title,
                                 trait = trait,
                                 output = output),
                            dotArgs[!(names(dotArgs) == "parCIsDat")]))

    }
  }
  invisible(p)
}

#' Create a plot with effect sizes
#'
#' Create a plot of QTL effect sizes and standard errors for each parent for
#' selected snps.
#'
#' @keywords internal
parCIsPlot <- function(parCIsDat,
                       xLab = "Effect",
                       yLab = "Parents",
                       title = NULL,
                       trait = NULL,
                       ...,
                       output = TRUE) {
  parCIsDat[["sign"]] <- as.factor(
    ifelse(parCIsDat[["effect"]] - 2 * parCIsDat[["seEffect"]] < 0 &
             parCIsDat[["effect"]] + 2 * parCIsDat[["seEffect"]] > 0,
           0, ifelse(parCIsDat[["effect"]] > 0,  1, 2)))
  parCIsDat[["snp"]] <- factor(parCIsDat[["snp"]],
                               levels = unique(parCIsDat[["snp"]]))
  ## Create title.
  if (is.null(title)) {
    title <- paste("Parental effects at QTLs for", trait)
  }
  if (nrow(parCIsDat) > 0) {
    p <- ggplot2::ggplot(parCIsDat,
                         ggplot2::aes(x = .data[["effect"]],
                                      y = .data[["parent"]],
                                      color = .data[["sign"]])) +
      ggplot2::geom_point(color = "black") +
      ggplot2::geom_errorbarh(ggplot2::aes(xmax = .data[["effect"]] + 2 * .data[["seEffect"]],
                                           xmin = .data[["effect"]] - 2 * .data[["seEffect"]]),
                              width = 0.2) +
      ## Trick to assure symmetrical x-axis.
      ggplot2::geom_blank(ggplot2::aes(xmin = - .data[["effect"]] - 2 * .data[["seEffect"]],
                                       xmax = - .data[["effect"]] + 2 * .data[["seEffect"]])) +
      ggplot2::geom_vline(xintercept = 0, linetype = "dashed") +
      ggplot2::labs(x = xLab, y = yLab, title = title) +
      ggplot2::facet_wrap(ggplot2::vars(.data[["snp"]])) +
      ggplot2::scale_y_discrete(limits = rev) +
      ggplot2::scale_color_manual(values = c("0" = "black", "1" = "red",
                                             "2" = "blue"),
                                  guide = "none") +
      ggplot2::theme(panel.background = ggplot2::element_blank(),
                     plot.background = ggplot2::element_blank(),
                     strip.background = ggplot2::element_blank(),
                     panel.border = ggplot2::element_rect(fill = NA,
                                                          color = "black",
                                                          linewidth = 0.5,
                                                          linetype = "solid"),
                     plot.title = ggplot2::element_text(hjust = 0.5))
  }
  if (output) {
    plot(p)
  }
  invisible(p)
}

#' IBD calculation for multi parental populations
#'
#' IBD calculation for multi parental populations. Per cross IBD probabilities
#' are calculated using \code{calcIBD} in the statgenIBD package. These
#' probabilities are combined with optional phenotypic data and stored in a
#' single object of class \code{gDataMPP}.
#'
#' IBD probabilities can be calculated for many different types of populations.
#' In the following table all supported populations are listed. Note that the
#' value of x in the population types is variable, with its maximum value
#' depicted in the last column.
#'
#' | __Population type__ | __Cross__ | __Description__ | __max. x__ |
#' | ------ | ----------------- | -------------------------------------- | --- |
#' | DH | biparental | doubled haploid population | |
#' | Fx | biparental | Fx population (F1, followed by x-1 generations of selfing) | 8 |
#' | FxDH | biparental | Fx, followed by DH generation | 8 |
#' | BCx | biparental | backcross, second parent is recurrent parent | 9 |
#' | BCxDH | biparental | BCx, followed by DH generation | 9 |
#' | BC1Sx | biparental | BC1, followed by x generations of selfing | 7 |
#' | BC1SxDH | biparental | BC1, followed by x generations of selfing and DH | 6 |
#' | C3 | three-way | three way cross: (AxB) x C |  |
#' | C3DH | three-way | C3, followed by DH generation |  |
#' | C3Sx | three-way | C3, followed by x generations of selfing | 7 |
#' | C3SxDH | three-way | C3, followed by x generations of selfing and DH generation | 6 |
#' | C4 | four-way | four-way cross: (AxB) x (CxD)	| |
#' | C4DH | four-way | C4, followed by DH generation |  |
#' | C4Sx | four-way | C4, followed by x generations of selfing | 6 |
#' | C4SxDH | four-way | C4, followed by x generations of selfing and DH generation | 6 |
#'
#' @param crossNames A character vector, the names of the crosses.
#' @param markerFiles A character vector indicating the locations of the files
#' with genotypic information for the populations. The files should be in
#' tab-delimited format with a header containing marker names.
#' @param pheno A data.frame or a list of data.frames with phenotypic data,
#' with genotypes in the first column \code{genotype} and traits in the
#' following columns. The trait columns should be numerical columns only.
#' A list of data.frames can be used for replications, i.e. different
#' trials.
#' @param popType A character string indicating the type of population. One of
#' DH, Fx, FxDH, BCx, BCxDH, BC1Sx, BC1SxDH, C3, C3DH, C3Sx, C3SxDH, C4, C4DH,
#' C4Sx, C4SxDH (see Details).
#' @param mapFile A character string indicating the location of the map file
#' for the population. The file should be in tab-delimited format. It should
#' consist of exactly three columns, marker, chromosome and position. There
#' should be no header. The positions in the file should be in centimorgan.
#' @param evalDist A numeric value, the maximum distance in cM between
#' evaluation points.
#' @param grid Should the extra markers that are added to assure the a
#' maximum distince of \code{evalDist} be on a grid (\code{TRUE}) or in between
#' marker existing marker positions (\code{FALSE}).
#' @param verbose Should progress be printed?
#'
#' @returns An object of class \code{gDataMPP} with the following components:
#' \item{\code{map}}{a data.frame containing map data. Map is sorted by
#' chromosome and position.}
#' \item{\code{markers}}{a 3D matrix containing IBD probabilities.}
#' \item{\code{pheno}}{data.frame or list of data.frames containing phenotypic
#' data.}
#' \item{\code{kinship}}{a kinship matrix.}
#' \item{\code{covar}}{a data.frame with extra covariates (including the
#' name of the cross).}
#'
#' @examples
#' ## Read phenotypic data.
#' pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
#'                                package = "statgenMPP"))
#' ## Rename first column to genotype.
#' colnames(pheno)[1] <- "genotype"
#'
#'
#' ## Compute IBD probabilities for simulated population - AxB, AxC.
#' ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
#'                   markerFiles = c(system.file("extdata/multipop", "AxB.txt",
#'                                               package = "statgenMPP"),
#'                                   system.file("extdata/multipop", "AxC.txt",
#'                                               package = "statgenMPP")),
#'                   pheno = pheno,
#'                   popType = "F4DH",
#'                   mapFile = system.file("extdata/multipop", "mapfile.txt",
#'                                         package = "statgenMPP"),
#'                   evalDist = 5)
#'
#' summary(ABC)
#'
#' @importFrom utils read.delim packageVersion
#' @importFrom stats setNames
#' @export
calcIBDMPP <- function(crossNames,
                       markerFiles,
                       pheno,
                       popType,
                       mapFile,
                       evalDist,
                       grid = TRUE,
                       verbose = FALSE) {
  ## Checks.
  if (!is.character(crossNames)) {
    stop("crossNames should be a character vector.\n")
  }
  if (!is.character(markerFiles)) {
    stop("markerFiles should be a character vector.\n")
  }
  if (length(crossNames) != length(markerFiles)) {
    stop("crossNames and markerFiles should have the same length.\n")
  }
  missFiles <- markerFiles[!file.exists(markerFiles)]
  if (length(missFiles) > 0) {
    stop("The following files don't exist: \n",
         paste(missFiles, collapse = ", "))
  }
  if (!is.null(pheno) && !inherits(pheno, "data.frame") &&
      !inherits(pheno, "list")) {
    stop("pheno should be a data.frame or a list of data.frames.\n")
  }
  if (inherits(pheno, "data.frame") && !hasName(x = pheno, name = "genotype")) {
    stop("pheno should have a column genotype.\n")
  }
  if (inherits(pheno, "list")) {
    if (!all(sapply(X = pheno, FUN = inherits, "data.frame"))) {
      stop("pheno should be a data.frame or a list of data.frames.\n")
    }
    if (!all(sapply(X = pheno, FUN = function(x) {
      hasName(x = x, name = "genotype")
    }))) {
      stop("All data.frames in pheno should have a column genotype.\n")
    }
  }
  if (!is.character(popType) || length(popType) > 1) {
    stop("popType should be a character string of length 1.\n")
  }
  if (!is.character(mapFile) || length(mapFile) > 1) {
    stop("mapFile should be a character string of length 1.\n")
  }
  if (!file.exists(mapFile)) {
    stop("mapFile doesn't exist.\n")
  }
  if (!is.numeric(evalDist) || length(evalDist) > 1 || evalDist < 0) {
    stop("evalDist should be a positive numerical value.\n")
  }
  ## Get number of crosses.
  nCross <- length(markerFiles)
  ## Calculate IBD probabilities per cross.
  crossIBD <- lapply(X = seq_len(nCross), FUN = function(i) {
    if (verbose) {
      cat(paste0("calculating IBD in cross: ", crossNames[i], ".\n"))
    }
    statgenIBD::calcIBD(popType = popType,
                        markerFile = markerFiles[i],
                        mapFile = mapFile,
                        evalDist = evalDist,
                        grid = grid,
                        verbose = verbose)
  })
  ## Concatenate results.
  crossIBD <- do.call(what = `c`, args = crossIBD)
  ## Replace cross names by cross names provided in input.
  if (nCross == 1) {
    covar <- data.frame(cross = rep(x = crossNames,
                                    times = nrow(crossIBD$markers)),
                        row.names = rownames(crossIBD$markers))
  } else {
    crossDat <- data.frame(cross = paste0("cross", seq_len(nCross)),
                           crossName = crossNames)
    covar <- attr(crossIBD, "genoCross")
    covar[["cross"]] <-
      crossDat[["crossName"]][match(x = covar[["cross"]],
                                    table = crossDat[["cross"]])]
    row.names(covar) <- covar[["geno"]]
    covar <- covar[, colnames(covar) != "geno", drop = FALSE]
  }
  ## Bind phenotypic data.
  if (inherits(pheno, "list")) {
    phenoTot <- do.call(what = rbind, args = pheno)
  } else {
    phenoTot <- pheno
  }
  ## Get number of parents.
  parents <- crossIBD$parents
  nPar <- length(parents)
  ## Construct empty marker matrix.
  ## Get marker names.
  markerNames <- colnames(crossIBD$markers)
  markers <- array(NA_real_, dim = c(dim(crossIBD$markers)[1:2], nPar),
                   dimnames = c(dimnames(crossIBD$markers)[1:2],
                                list(parents)))
  ## Fill marker matrix.
  for (i in seq_along(markerNames)) {
    markers[, i, ] <- markers3DtoMat(markers = crossIBD$markers,
                                     parents = parents,
                                     markerSel = markerNames[i])
  }
  ## Read original map.
  mapOrig <- read.delim(mapFile, header = FALSE)
  rownames(mapOrig) <- mapOrig[[1]]
  mapOrig <- mapOrig[, -1]
  colnames(mapOrig) <- c("chr", "pos")
  ## Create gDataMPP object.
  MPPobj <- createGDataMPPInternal(geno = markers,
                                   map = crossIBD$map,
                                   pheno = phenoTot,
                                   covar = covar)
  attr(x = MPPobj, which = "popType") <- crossIBD$popType
  attr(x = MPPobj, which = "pedigree") <- crossIBD$pedigree
  attr(x = MPPobj, which = "genoCross") <- attr(x = crossIBD, which = "genoCross")
  attr(x = MPPobj, which = "mapOrig") <- mapOrig
  return(MPPobj)
}

### Test selQTLMPP

## Define input files.

markerFiles <- c(system.file("extdata/multipop", "AxB.txt",
                             package = "statgenMPP"),
                 system.file("extdata/multipop", "AxC.txt",
                             package = "statgenMPP"))
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")

pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                package = "statgenMPP"))

## Compute IBD probabilities.

# High evaldist for faster computations.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = markerFiles,
                  pheno = pheno, popType = "F4DH",
                  mapFile = mapFile, evalDist = 25)

## QTL Detection.
ABC_MQM <- selQTLMPP(MPPobj = ABC, trait = "yield")

## Summary.
sumABC <- capture.output(summary(ABC_MQM))
expect_true("Number of QTLs: 3 " %in% sumABC)
expect_true("Threshold: 3 " %in% sumABC)

## QTLProfile is a call to manhattan plot in statgenGWAS. Not much to test.
p1 <- plot(ABC_MQM, plotType = "QTLProfile")
expect_inherits(p1, "ggplot")

## QTLRegion is a call to geneticMap plot in statgenGWAS. Not much to test.
p2 <- plot(ABC_MQM, plotType = "QTLRegion")
expect_inherits(p2, "ggplot")

## Parental effects.
p3 <- plot(ABC_MQM, plotType = "parEffs")
expect_inherits(p3, "ggplot")

## Extended QTL profile.
p4 <- plot(ABC_MQM, plotType = "QTLProfileExt")
expect_inherits(p4, "gtable")

## Confidence intervals around parental effects.
p5 <- plot(ABC_MQM, plotType = "parCIs")
expect_inherits(p5, "ggplot")

### Test selQTLMPP

## Define input files.

markerFiles <- c(system.file("extdata/multipop", "AxB.txt",
                             package = "statgenMPP"),
                 system.file("extdata/multipop", "AxC.txt",
                             package = "statgenMPP"))
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")

pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                package = "statgenMPP"))
colnames(pheno)[1] <- "genotype"

## Compute IBD probabilities.

# High evaldist for faster computations.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = markerFiles,
                  pheno = pheno, popType = "F4DH",
                  mapFile = mapFile, evalDist = 25)

## Check inputs.

expect_error(selQTLMPP(MPPobj = 1),
             "MPPobj should be an object of class gDataMPP")
expect_error(selQTLMPP(MPPobj = ABC, trait = 1),
             "trait should be a character string of length one present in")
expect_error(selQTLMPP(MPPobj = ABC, trait = "tst"),
             "trait should be a character string of length one present in")
expect_error(selQTLMPP(MPPobj = ABC, trait = "yield", QTLwindow = "1"),
             "QTLwindow should be a positive numerical value")
expect_error(selQTLMPP(MPPobj = ABC, trait = "yield", QTLwindow = c(1, 2)),
             "QTLwindow should be a positive numerical value")
expect_error(selQTLMPP(MPPobj = ABC, trait = "yield", threshold = "1"),
             "threshold should be a positive numerical value")
expect_error(selQTLMPP(MPPobj = ABC, trait = "yield", threshold = c(1, 2)),
             "threshold should be a positive numerical value")
expect_error(selQTLMPP(MPPobj = ABC, trait = "yield", maxCofactors = "1"),
             "maxCofactors should be a positive numerical value")
expect_error(selQTLMPP(MPPobj = ABC, trait = "yield", maxCofactors = c(1, 2)),
             "maxCofactors should be a positive numerical value")

ABC_MQM_max <- selQTLMPP(MPPobj = ABC, trait = "yield")

## General structure.
expect_inherits(ABC_MQM_max, "QTLMPP")
expect_inherits(ABC_MQM_max, "GWAS")

expect_equal(names(ABC_MQM_max),
             c("GWAResult", "signSnp", "kinship", "thr", "GWASInfo"))
expect_equal_to_reference(ABC_MQM_max, "ABC_MQM_max", tolerance = 1e-2)

## Option verbose.
expect_stdout(selQTLMPP(MPPobj = ABC, trait = "yield", maxCofactors = 1,
                        verbose = TRUE),
              "QTL scan for trait yield, 0 cofactors")

## Option kin.
ABC_SQM_kin <- selQTLMPP(MPPobj = ABC, trait = "yield", computeKin = TRUE,
                         maxCofactors = 0)
expect_inherits(ABC_SQM_kin$kinship, "list")
expect_equal(length(ABC_SQM_kin$kinship), 5)

ABC_SQM_kin2 <- selQTLMPP(MPPobj = ABC, trait = "yield",
                          K = ABC_SQM_kin$kinship, maxCofactors = 0)

expect_equal(ABC_SQM_kin, ABC_SQM_kin2)

### Test gDataMPP

## Define input files.

markerFiles <- c(system.file("extdata/multipop", "AxB.txt",
                             package = "statgenMPP"),
                 system.file("extdata/multipop", "AxC.txt",
                             package = "statgenMPP"))
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")

pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                package = "statgenMPP"))

## Compute IBD probabilities.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = markerFiles,
                  pheno = pheno, popType = "F4DH",
                  mapFile = mapFile, evalDist = 5)

## Check summary.

sumABC <- summary(ABC)

expect_inherits(sumABC, "summary.gDataMPP")
expect_equal(names(sumABC), c("mapSum", "markerSum", "phenoSum", "covarSum"))

## Check printed output.
sumABCprnt <- capture.output(summary(ABC))
expect_true("\tNumber of traits: 1 " %in% sumABCprnt)
expect_true("\tTraitnames: yield " %in% sumABCprnt)
expect_true(" AxB:100  " %in% sumABCprnt)
expect_true(" AxC: 80  " %in% sumABCprnt)

# Function is a copy of function from statgenGWAS except for markerSum bit.
# Only check that.

expect_inherits(sumABC$markerSum, "list")
expect_equal(names(sumABC$markerSum), c("nMarkers", "nGeno", "markerContent"))
expect_equal(sumABC$markerSum$markerContent, c(`parents:` = "A, B, C"))

## Check plot.

# All plots are called directly from statgenGWAS or statgenIBD.
# Not much can be tested here.

# genMap
expect_error(plot(ABC, highlight = "EXT_3_31"),
             "The following highlight markers are not in map")
expect_silent(p1 <- plot(ABC, plotType = "genMap"))
p1a <- plot(ABC, highlight = "EXT_3_30")

expect_equal(length(p1a$layers), length(p1$layers) + 2)


expect_silent(p2 <- plot(ABC, plotType = "singleGeno", genotype = "AxB0001"))
expect_silent(p3 <- plot(ABC, plotType = "allGeno"))
expect_silent(p4 <- plot(ABC, plotType = "pedigree"))

### Test calcIBDMPP

## Define input files.

markerFiles <- c(system.file("extdata/multipop", "AxB.txt",
                             package = "statgenMPP"),
                 system.file("extdata/multipop", "AxC.txt",
                             package = "statgenMPP"))
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")

pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                package = "statgenMPP"))

phenoLst <- list(pheno[1:100, ], pheno[101:180, ])

## Checks for correct input.
# crossNames.
expect_error(calcIBDMPP(crossNames = 1),
             "crossNames should be a character vector")
expect_error(calcIBDMPP(crossNames = "AxB", markerFiles = markerFiles),
             "crossNames and markerFiles should have the same length")

# markerFiles.
expect_error(calcIBDMPP(crossNames = "AxB", markerFiles = 1),
             "markerFiles should be a character vector")
expect_error(calcIBDMPP(crossNames = "AxB", markerFiles = "tst"),
             "The following files don't exist")

# pheno
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = 1),
             "pheno should be a data.frame or a list of data.frames")
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = pheno[, 2, drop = FALSE]),
             "pheno should have a column genotype")

# pheno list.
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = list(1, 2)),
             "pheno should be a data.frame or a list of data.frames")
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = list(pheno[1:100, 2, drop = FALSE],
                                     pheno[101:180, ])),
             "All data.frames in pheno should have a column genotype")

# popType
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = pheno, popType = 1),
             "popType should be a character string of length 1")

# mapFile
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = pheno, popType = "F4DH", mapFile = 1),
             "mapFile should be a character string of length 1")
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = pheno, popType = "F4DH", mapFile = "tst.txt"),
             "mapFile doesn't exist")

# evalDist
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = pheno, popType = "F4DH", mapFile = mapFile,
                        evalDist = "a"),
             "evalDist should be a positive numerical value")
expect_error(calcIBDMPP(crossNames = c("AxB", "AxC"), markerFiles = markerFiles,
                        pheno = pheno, popType = "F4DH", mapFile = mapFile,
                        evalDist = c(1, 2)),
             "evalDist should be a positive numerical value")

## Large evalDist for faster computations.
expect_silent(ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                                markerFiles = markerFiles,
                                pheno = pheno, popType = "F4DH",
                                mapFile = mapFile, evalDist = 25))
expect_silent(ABCPhenoLst <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                                        markerFiles = markerFiles,
                                        pheno = phenoLst, popType = "F4DH",
                                        mapFile = mapFile, evalDist = 25))

## General structure.
expect_inherits(ABC, "gDataMPP")

expect_inherits(ABC$map, "data.frame")
expect_equal(dim(ABC$map), c(15, 2))

expect_inherits(ABC$markers, "array")
expect_equal(dim(ABC$markers), c(180, 15, 3))

expect_inherits(ABC$pheno$pheno, "data.frame")

expect_inherits(ABC$covar, "data.frame")

expect_inherits(attr(ABC, "genoCross"), "data.frame")
expect_inherits(attr(ABC, "pedigree"), "data.frame")

expect_equal(ABC, ABCPhenoLst)

## Verbose = TRUE
printOut <- capture.output(
  ABCVerb <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                        markerFiles = markerFiles,
                        pheno = pheno, popType = "F4DH",
                        mapFile = mapFile, evalDist = 25,
                        verbose = TRUE))
expect_true("calculating IBD in cross: AxB." %in% printOut)
expect_true("reading data .............." %in% printOut)
expect_true("analysis of family ........" %in% printOut)

## Single cross.
ABCOne <- calcIBDMPP(crossNames = "AxB",
                     markerFiles = markerFiles[1],
                     pheno = pheno, popType = "F4DH",
                     mapFile = mapFile, evalDist = 25)
expect_true(all(ABCOne$covar$cross == "AxB"))

#' Compute kinship matrix for IBD probabilities
#'
#' Compute a kinship matrix or a list of chromosome specific kinship matrices
#' for a 3D array of IBD probabilities. The kinship matrix is computed by
#' averaging \eqn{Z Z^t} over all markers, where \eqn{Z} is the genotype x
#' parents matrix for the marker. If \code{chrSpecific = TRUE} chromosome
#' specific kinship matrices are computed for each chromosome based only on the
#' markers on all other chromosomes.
#'
#' @param markers An n x m x p array with IBD probabilities with genotypes in
#' the rows (n), markers in the columns (m), and parents in the 3rd dimension
#' (p).
#' @param map A data.frame with columns \code{chr} for chromosome and
#' \code{pos} for position. Positions should be in centimorgan (cM). They
#' should not be cumulative over the chromosomes. Other columns are ignored.
#' Marker names should be in the row names. These should match the marker names
#' in the input file. Only required if \code{chrSpecific = TRUE}.
#' @param chrSpecific Should chromosome specific kinship matrices be
#' computed?
#'
#' @returns A kinship matrix or a list of chromosome specific kinship matrices.
#'
#' @examples
#' ## Read phenotypic data.
#' pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
#'                                package = "statgenMPP"))
#' ## Rename first column to genotype.
#' colnames(pheno)[1] <- "genotype"
#'
#'
#' ## Compute IBD probabilities for simulated population - AxB, AxC.
#' ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
#'                   markerFiles = c(system.file("extdata/multipop", "AxB.txt",
#'                                               package = "statgenMPP"),
#'                                   system.file("extdata/multipop", "AxC.txt",
#'                                               package = "statgenMPP")),
#'                   pheno = pheno,
#'                   popType = "F4DH",
#'                   mapFile = system.file("extdata/multipop", "mapfile.txt",
#'                                         package = "statgenMPP"),
#'                   evalDist = 5)
#'
#' ## Compute chromosome specific kinship matrices.
#' KChrSpec <- kinshipIBD(markers = ABC$markers, map = ABC$map)
#'
#' @export
kinshipIBD <- function(markers,
                       map = NULL,
                       chrSpecific = TRUE) {
  if (!is.array(markers) && !length(dim(markers)) == 3) {
    stop("markers should be a 3 dimensional array.\n")
  }
  if (chrSpecific) {
    if (is.null(map) || !is.data.frame(map)) {
      stop("map should be a data.frame.\n")
    }
    if (!all(hasName(x = map, name = c("chr", "pos")))) {
      ## chr and pos are obligatory cols.
      stop("chr and pos should be columns in map.\n")
    }
    K <- sapply(X = as.character(unique(map$chr)), FUN = function(chr) {
      mrkNamesNonChr <- rownames(map)[map$chr != chr]
      mrkNonChr <- apply(markers[, mrkNamesNonChr, ], MARGIN = 2,
                         FUN = tcrossprod, simplify = FALSE)
      KChr <- Reduce(`+`, mrkNonChr) / length(mrkNonChr)
      rownames(KChr) <- colnames(KChr) <- rownames(markers)
      return(KChr)
    }, simplify = FALSE)
  } else {
    mrk <- apply(markers, MARGIN = 2, FUN = tcrossprod, simplify = FALSE)
    K <- Reduce(`+`, mrk) / ncol(markers)
    rownames(K) <- colnames(K) <- rownames(markers)
  }
  return(K)
}

### Test kinshipIBD

set.seed(1234)
## Create marker data.
markers <- array(runif(45), dim = c(3, 5, 3),
                 dimnames = list(paste0("G", 1:3), paste0("M", 1:5),
                                 paste0("P", 1:3)))
markers <- simplify2array(apply(X = markers, MARGIN = 2, FUN = function(x) {
  x / rowSums(x)
}, simplify = FALSE))
markers <- aperm(markers, c(1, 3, 2))

## Construct map.
map <- data.frame(chr = c(1, 1, 2, 2, 2), pos = 1:5,
                  row.names = paste0("M", 1:5))


## General checks

expect_error(kinshipIBD(markers = 1),
             "markers should be a 3 dimensional array")
expect_error(kinshipIBD(markers = markers, map = 1),
             "map should be a data.frame")
expect_error(kinshipIBD(markers = markers, map = map[, 1, drop = FALSE]),
             "chr and pos should be columns in map")

## Chromosome specific
KChrSpec <- kinshipIBD(markers = markers, map = map)

expect_inherits(KChrSpec, "list")
expect_equal(length(KChrSpec), 2)
expect_equal(names(KChrSpec), c("1", "2"))
expect_inherits(KChrSpec[[1]], "matrix")
expect_equal(dim(KChrSpec[[1]]), c(3, 3))

expect_equal(as.numeric(KChrSpec[[1]]),
             c(0.450980778777623, 0.318347657907409, 0.274145151365274,
               0.318347657907409, 0.342980515400794, 0.343517588921626,
               0.274145151365274, 0.343517588921626, 0.423994558163868))

## Non chromosome specific

K <- kinshipIBD(markers = markers, chrSpecific = FALSE)

expect_inherits(K, "matrix")
expect_equal(dim(K), c(3, 3))

expect_equal(as.numeric(K),
             c(0.442181446699766, 0.323298364111491, 0.291059186451823,
               0.323298364111491, 0.386092061743138, 0.353213553269824,
               0.291059186451823, 0.353213553269824, 0.404976882045675))

#' cofactor selection based on the QTL windowsize
#'
#' @keywords internal
calcDistance <- function(map,
                         m1,
                         m2) {
  dist <- ifelse(map[m1, "chr"] == map[m2, "chr"],
                 abs(map[m1, "pos"] - map[m2, "pos"]),
                 Inf)
  return(dist)
}

#' @keywords internal
selectCofactors <- function(map,
                            marker,
                            cofactors,
                            QTLwindow) {
  if (length(cofactors) == 0) return(NULL)
  minDist <- 0.5 * QTLwindow
  dist <- sapply(cofactors, function(x) calcDistance(map, marker, x))
  if (min(dist) <= minDist) {
    cofactors <- cofactors[-which.min(dist)]
    if (length(cofactors) == 0) return(NULL)
  }
  return(cofactors)
}

#' @importFrom stats aggregate
#'
#' @keywords internal
plotIntermediateScan <- function(dat,
                                 threshold,
                                 cofactors,
                                 trait) {
  ## Construct title.
  title <- paste("QTL-profile for trait ", trait)
  if (length(cofactors) == 0) {
    title <- paste0(title, ", no cofactors")
  } else if (length(cofactors) == 1) {
    title <- paste0(title, ", one cofactor")
  } else {
    title <- paste0(title,", ", length(cofactors)," cofactors")
  }
  map <- dat[c("chr", "pos")]
  map[["chr"]] <- factor(map[["chr"]], levels = unique(map[["chr"]]))
  ## Get the boundary for each of the chromosomes.
  ## Has to be converted to numeric to avoid integer overflow in the next step.
  chrBnd <- aggregate(x = map[["pos"]], by = list(map[["chr"]]),
                      FUN = function(p) {as.numeric(max(p))})
  ## Compute cumulative positions.
  addPos <- data.frame(chr = chrBnd[, 1],
                       add = c(0, cumsum(chrBnd[, 2]))[1:nrow(chrBnd)],
                       stringsAsFactors = FALSE)
  map <- merge(map, addPos, by = "chr")
  map[["cumPos"]] <- map[["pos"]] + map[["add"]]
  manhattanPlot(xValues = map[["cumPos"]],
                yValues = dat$minlog10p,
                plotType = "lines",
                map = map,
                yThr = threshold,
                title = title)
}

#' Helper function for creating summaries that always display NA.
#'
#' @noRd
#' @keywords internal
summaryNA <- function(dat) {
  if (!any(is.na(dat))) {
    return(c(summary(dat), "NA's" = 0))
  } else{
    return(summary(dat))
  }
}

#' Helper function for accessing parallel computing functions.
#'
#' @noRd
#' @importFrom foreach `%do%` `%dopar%`
#' @keywords internal
getOper <- function(x) {
  if (x) {
    `%dopar%`
  } else {
    `%do%`
  }
}

#' Create a matrix plot
#'
#' Create a matrix plot of effect sizes and directions per parent for
#' significant snps.
#'
#' @import grDevices graphics
#'
#' @keywords internal
effectPlot <- function(effectDat,
                       signSnp,
                       map,
                       xLab = "Chromosomes",
                       yLab = "Parents",
                       chrBoundaries,
                       title = NULL,
                       trait = NULL,
                       ...,
                       output = TRUE) {
  ## Extract central chromosome positions from map.
  ## Differentiate cases to deal with character chromosomes.
  if (is.numeric(map$chr)) {
    chrs <- as.numeric(levels(as.factor(map$chr)))
  } else {
    chrs <- levels(as.factor(map$chr))
  }
  ## Recompute cumulative positions.
  addPos <- data.frame(chr = chrBoundaries[, 1],
                       add = c(0, cumsum(chrBoundaries[, 2] + 5))[1:nrow(chrBoundaries)],
                       stringsAsFactors = FALSE)
  map <- merge(map[, !colnames(map) %in% c("add", "cumPos")],
               addPos, by = "chr")
  map[["cumPos"]] <- map[["pos"]] + map[["add"]]
  map[["width"]] <- unlist(
    aggregate(x = map[["pos"]], by = list(map[["chr"]]),
              FUN = function(x) {
                diff0 <- diff(c(0, x), lag = 2) / 2
                return(c(diff0, tail(diff0, 1)))
              }, simplify = FALSE)[, 2])
  ## Compute postions of labels for chromosomes.
  xMarks <- aggregate(x = map[["cumPos"]], by = list(map[["chr"]]),
                      FUN = function(x) {
                        min(x) + (max(x) - min(x)) / 2
                      })[, 2]
  ## Compute chromosome boundaries.
  chrBnd <- c(0, aggregate(x = map[["cumPos"]],
                           by = list(map[["chr"]]), FUN = max)[[2]])
  chrBnd <- chrBnd + c(0, aggregate(x = map[["width"]], by = list(map[["chr"]]),
                                    FUN = tail, 1)[[2]])
  ## Add cumulative position from map to effects.
  parEffData <- merge(effectDat, map[, c("snp", "cumPos", "width")],
                      by = "snp", sort = FALSE)
  ## Only plotting the effects for significant SNPs. Remove all others.
  parEffData <- parEffData[interaction(parEffData[["snp"]], parEffData[["trait"]]) %in%
                             interaction(signSnp[["snp"]], signSnp[["trait"]]), ]
  if (nrow(parEffData) > 0) {
    maxVal <- max(abs(parEffData$effect), na.rm = TRUE)
  }
  ## Create title.
  if (is.null(title)) {
    title <- paste("Parental effects at QTLs for", trait)
  }
  p <- ggplot2::ggplot() +
    ggplot2::scale_x_continuous(breaks = xMarks, labels = chrs,
                                expand = c(0, 0)) +
    ggplot2::labs(title = title, x = xLab, y = yLab) +
    ggplot2::theme(panel.background = ggplot2::element_blank(),
                   plot.background = ggplot2::element_blank(),
                   strip.background = ggplot2::element_blank(),
                   panel.border = ggplot2::element_rect(fill = NA,
                                                        color = "black",
                                                        linewidth = 0.5,
                                                        linetype = "solid"),
                   plot.title = ggplot2::element_text(hjust = 0.5))
  if (nrow(parEffData) > 0) {
    p <- p +
      ggplot2::geom_tile(ggplot2::aes(x = .data[["cumPos"]],
                                      y = .data[["trait"]],
                                      fill = .data[["effect"]],
                                      width = .data[["width"]]),
                         height = 1, data = parEffData) +
      ggplot2::scale_fill_gradientn(colors = c("blue", "cyan", "white",
                                               "yellow","red"),
                                    values = scales::rescale(c(-1,
                                                               - sqrt(.Machine$double.eps),
                                                               0,
                                                               sqrt(.Machine$double.eps),
                                                               1)),
                                    limits = c(-maxVal, maxVal),
                                    na.value = "white") +
      ggplot2::scale_y_discrete(expand = c(0, 0), limits = rev)
  }
  p <- p + ggplot2::geom_vline(xintercept = chrBnd, color = "grey20",
                               lty = 2, linewidth = 0.3)
  if (output) {
    plot(p)
  }
  invisible(p)
}

#' S3 Class gDataMPP
#'
#' \code{createGDataMPPInternal} creates an object of S3 class gDataMPP with genotypic and
#' phenotypic data for usage in further analysis. All input to the function is
#' optional, however at least one input should be provided. It is possible to
#' provide an existing \code{gDataMPP} object as additional input in which case
#' data is added to this object. Existing data will be overwritten with a
#' warning.
#'
#' @param gDataMPP An optional gDataMPP object to be modified. If \code{NULL}, a new
#' gDataMPP object is created.
#' @param geno A matrix or data.frame with genotypes in the rows and markers in
#' the columns. A matrix from the \code{matrix} in the base package may be
#' provided as well as as matrix from the Matrix package.\cr
#' A three dimensional array of probabilities may be provided as well with
#' genotypes in the first, markers in the second and alleles in the third
#' dimension.\cr
#' If no row names are provided, they are taken from \code{pheno} (if supplied and
#' dimension matches). If no column names are provided, the row names
#' from \code{map} are used (if supplied and dimension matches).
#' @param map A data.frame with columns \code{chr} for chromosome and
#' \code{pos} for position. Positions can be in base pair (bp) or centimorgan (cM). They
#' should not be cumulative over the chromosomes. Other columns are ignored.
#' Marker names should be in the row names. These should match the marker names
#' in \code{geno} (if supplied).
#' @param kin A kinship matrix or list of kinship matrices with genotype in
#' rows and colums. These matrices can be from the \code{matrix} class, as
#' defined in the base package, or from the \code{dsyMatrix} class, the class
#' of symmetric matrices in the Matrix package.\cr
#' The genotypes should be identical to the genotypes in \code{geno}.\cr
#' If a list of kinship matrices is provided these are supposed to be
#' chromosome specific matrices. In that case their names should match
#' the names of the chromosomes in \code{map}. If no names are
#' provided, the number of matrices should match the number of chromosomes
#' in \code{map} in which case default names are provided.
#' @param pheno A data.frame or a list of data.frames with phenotypic data,
#' with genotypes in the first column \code{genotype} and traits in the
#' following columns. The trait columns should be numerical columns only.
#' A list of data.frames can be used for replications, i.e. different
#' environments.
#' @param covar A data.frame with extra covariates per genotype. Genotypes
#' should be in the rows.
#'
#' @returns An object of class \code{gData} with the following components:
#' \item{\code{map}}{a data.frame containing map data. Map is sorted by
#' chromosome and position.}
#' \item{\code{markers}}{a sparse matrix from the Matrix package containing
#' marker information in case of numerical genotypic data, a standard matrix
#' otherwise.\cr
#' If \code{geno} is a three dimensional array, \code{markers} is a three dimensional
#' array as well.}
#' \item{\code{pheno}}{a list of data.frames containing phenotypic data.}
#' \item{\code{kinship}}{a kinship matrix of class \code{dsyMatrix} from the
#'  Matrix package.}
#' \item{\code{covar}}{a data.frame with extra covariates.}
#'
#' @seealso \code{\link{summary.gDataMPP}}
#'
#' @examples
#' set.seed(1234)
#' ## Create genotypic data.
#' geno <- matrix(sample(x = c(0, 1, 2), size = 15, replace = TRUE), nrow = 3)
#' dimnames(geno) <- list(paste0("G", 1:3), paste0("M", 1:5))
#'
#' ## Construct map.
#' map <- data.frame(chr = c(1, 1, 2, 2, 2), pos = 1:5,
#'                   row.names = paste0("M", 1:5))
#'
#' ## Create phenotypic data.
#' pheno <- data.frame(paste0("G", 1:3),
#'                     matrix(rnorm(n = 12, mean = 50, sd = 5), nrow = 3),
#'                     stringsAsFactors = FALSE)
#' dimnames(pheno) = list(paste0("G", 1:3), c("genotype", paste0("T", 1:4)))
#'
#' ## Combine all data in gDataMPP object.
#' gDataMPP <- createGDataMPPInternal(geno = geno, map = map, pheno = pheno)
#' summary(gDataMPP)
#'
#' ## Construct covariate.
#' covar <- data.frame(C1 = c("a", "a", "b"), row.names = paste0("G", 1:3))
#'
#' ## Add covariates to previously created gDataMPP object.
#' gData2 <- createGDataMPPInternal(gDataMPP = gDataMPP, covar = covar)
#'
#' @noRd
#' @keywords internal
createGDataMPPInternal <- function(gDataMPP = NULL,
                                   geno = NULL,
                                   map = NULL,
                                   kin = NULL,
                                   pheno = NULL,
                                   covar = NULL) {
  ## Check gDataMPP
  if (!is.null(gDataMPP) && !inherits(gDataMPP, "gDataMPP")) {
    stop("Provided gDataMPP object should be of class gDataMPP.\n")
  }
  ## Check that at least one input argument, other than gData, is provided.
  if (is.null(geno) && is.null(map) && is.null(kin) && is.null(pheno) &&
      is.null(covar)) {
    stop("At least one of geno, map, kin, pheno and covar should be",
         "provided.\n")
  }
  if (is.null(geno) || length(dim(geno)) == 2) {
    gDataNw <- statgenGWAS::createGData(gData = gDataMPP, geno = geno, map = map,
                                        kin = kin, pheno = pheno, covar = covar)
  } else {
    if (!is.null(map) || !is.null(pheno)) {
      gDataNw <- statgenGWAS::createGData(gData = gDataMPP, map = map,
                                          pheno = pheno)
      map <- gDataNw$map
      pheno <- gDataNw$pheno
    } else {
      gDataNw <- structure(list(map = NULL, markers = NULL, pheno = NULL,
                                kinship = NULL, covar = NULL),
                           class = "gDataMPP")
    }
    ## Modify geno.
    if (!is.null(geno)) {
      ## The regular 2d case is covered by the function from statgenGWAS.
      ## Here only the 3d case has to be taken care of.
      if (!(is.array(geno) && length(dim(geno)) == 3)) {
        stop("geno should be a three-dimensional array.\n")
      }
      markers <- geno
      ## Check for row names in markers. If not available take them from pheno
      ## or use default names.
      if (all(rownames(markers) == as.character(1:nrow(markers)))) {
        if (is.null(pheno)) {
          ## Default names are constructed as g001, g002, etc. with the number
          ## of zeros dependent on the number of rows.
          rownames(markers) <-
            paste0("g", formatC(1:nrow(markers),
                                width = ceiling(log10(nrow(markers))),
                                flag = "0"))
          warning("geno contains no genotype names. Default names used.\n",
                  call. = FALSE)
        } else {
          ## Phenotypic data available. Try to copy names of genotypes from
          ## genotypic data. If dimensions don't match throw an error.
          if (nrow(pheno[[1]]) == nrow(markers)) {
            rownames(markers) <- rownames(pheno[[1]])
            warning("geno contains no genotype names. Names taken from pheno.\n",
                    call. = FALSE)
          } else {
            stop("geno contains no genotype names. Dimensions between ",
                 "geno and pheno differ.\n")
          }
        }
      } else {
        ## Sort alphabetically by genotypes.
        markers <- markers[order(rownames(markers)), , , drop = FALSE]
      }
      if (is.null(colnames(markers))) {
        ## Check for column names in markers. If not available take them from map.
        ## If map not available or dimensions don't match throw an error.
        if (is.null(map)) {
          stop("geno contains no marker names. Map not available.\n")
        }
        if (nrow(map) != ncol(markers)) {
          stop("geno contains no marker names. Dimensions between geno",
               "and map differ.\n")
        }
        colnames(markers) <- rownames(map)
        warning("geno contains no marker names. Names taken from map.\n",
                call. = FALSE)
      } else if (!is.null(map)) {
        ## Both markers and map available. Make sure markernames in markers match
        ## markernames in map. Remove non-matching markers from markers.
        ## Map may still contain markers that are not in markers.
        if (any(!colnames(markers) %in%
                rownames(map)[rownames(map) %in% colnames(markers)])) {
          warning("Not all markers in geno are in map. Extra markers ",
                  "will be removed.\n", call. = FALSE)
        }
        markers <- markers[, colnames(markers) %in% rownames(map), ,
                           drop = FALSE]
        ## Check that genotypes don't contain NA probabilities.
        ## This might occur in RABBIT output.
        ## Remove those genotypes.
        genoNA <- apply(X = markers, MARGIN = 1, function(x) {
          anyNA(x) | any(is.infinite(x))
        })
        if (any(genoNA)) {
          warning("The following genotypes have been removed because ",
                  "they have missing values for probabilities:",
                  paste(rownames(markers)[genoNA], collapse = ", "))
          markers <- markers[!genoNA, , ]
        }
        ## Check that probabilities for each marker sum to one.
        ## Always rescale values.
        ## Throw a warning if difference from one is too large.
        genoMrk <- setNames(as.data.frame(matrix(nrow = 0, ncol = 2)),
                            c("geno", "marker"))
        for (mrk in colnames(markers)) {
          mrkProbs <- rowSums(markers[, mrk, ], na.rm = TRUE)
          if (any(abs(mrkProbs - 1) > 1e-2)) {
            genoMrk <-
              rbind(genoMrk,
                    data.frame(geno = names(mrkProbs[abs(mrkProbs - 1) > 1e-2]),
                               marker = mrk, stringsAsFactors = FALSE))
          }
          markers[, mrk, ] <- markers[, mrk, ] / mrkProbs
        }
        if (nrow(genoMrk) > 0) {
          genoMrk$genoMarker <- paste(genoMrk$geno, "\t", genoMrk$marker)
          warning("Probabilities differ from 1 for the following",
                  "combinations of genotype and markers:\n",
                  paste(genoMrk$genoMarker, collapse = "\n"), call. = FALSE)
        }
      }
      if (!is.null(gDataMPP$markers)) {
        ## gDataMPP already contained a markers object. Overwrite with a warning.
        warning("existing geno will be overwritten.\n", call. = FALSE)
      }
      gDataNw$markers <- markers
    }
    if (!is.null(kin) || !is.null(covar)) {
      gDataNw <- suppressWarnings(statgenGWAS::createGData(gData = gDataNw,
                                                           kin = kin,
                                                           covar = covar))
    }
  }
  class(gDataNw) <- c("gDataMPP", class(gDataNw))
  return(gDataNw)
}


#' Summary function for the class \code{gDataMPP}
#'
#' Gives a summary for an object of S3 class \code{gDataMPP}.
#'
#' @param object An object of class \code{gDataMPP}.
#' @param ... Not used.
#' @param trials A vector of trials to include in the summary. These can
#' be either numeric indices or character names of list items in \code{pheno}.
#' If \code{NULL}, all trials are included.
#'
#' @returns A list with a most four components:
#' \describe{
#' \item{mapSum}{A list with number of markers and number of chromosomes in
#' the map.}
#' \item{markerSum}{A list with number of markers, number of genotypes and
#' the distribution of the values within the markers.}
#' \item{phenoSum}{A list of data.frames, one per trial with a summary of all
#' traits within the trial.}
#' \item{covarSum}{A list of data.frames, one per trial with a summary of all
#' covariates within the trial.}
#' }
#' All components are only present in the output if the corresponding content is
#' present in the \code{gDataMPP} object.
#'
#' @export
summary.gDataMPP <- function(object,
                             ...,
                             trials = NULL) {
  if (length(dim(object$markers)) == 2) {
    NextMethod(generic = "summary", object, ..., trials)
  } else {
    ## If trials is null set trials to all trials in pheno.
    if (is.null(trials)) {
      trials <- seq_along(object$pheno)
    }
    map <- object$map
    markers <- object$markers
    pheno <-  object$pheno
    covar <- object$covar
    totSum <- vector(mode = "list")
    if (!is.null(map)) {
      mapSum <- list(nMarkers = nrow(map), nChr = length(unique(map[["chr"]])))
      totSum$mapSum <- mapSum
    }
    if (!is.null(markers)) {
      markerSum <- list(nMarkers = ncol(markers), nGeno = nrow(markers),
                        markerContent = setNames(
                          paste(dimnames(markers)[[3]], collapse = ", "),
                          "parents:"))
      totSum$markerSum <- markerSum
    }
    if (!is.null(pheno)) {
      phenoSum <- sapply(X = names(pheno[trials]), FUN = function(trial) {
        trSum <- do.call(cbind, lapply(X = pheno[[trial]][, -1, drop = FALSE],
                                       FUN = summaryNA))
        attr(x = trSum, which = "nGeno") <-
          length(unique(pheno[[trial]][["genotype"]]))
        return(trSum)
      }, simplify = FALSE)
      totSum$phenoSum <- phenoSum
    }
    if (!is.null(covar)) {
      covarSum <- summary(covar, maxsum = 99)
      colnames(covarSum) <- ""
      totSum$covarSum <- covarSum
    }
    return(structure(totSum,
                     class = "summary.gDataMPP"))
  }
}

#' Printing summarized objects of class gData
#'
#' \code{print} method for object of class summary.gData created by summarizing
#' objects of class gData.
#'
#' @param x An object of class \code{summary.gData}.
#' @param ... Not used.
#'
#' @importFrom utils tail
#'
#' @noRd
#' @export
print.summary.gDataMPP <- function(x,
                                   ...) {
  if (!is.null(x$mapSum)) {
    cat("map\n")
    cat("\tNumber of markers:", x$mapSum$nMarkers, "\n")
    cat("\tNumber of chromosomes:", x$mapSum$nChr, "\n\n")
  }
  if (!is.null(x$markerSum)) {
    cat("markers\n")
    cat("\tNumber of markers:", x$markerSum$nMarkers, "\n")
    cat("\tNumber of genotypes:", x$markerSum$nGeno, "\n")
    cat("\tParents:", x$markerSum$markerContent, "\n")
  }
  if (!is.null(x$phenoSum)) {
    cat("pheno\n")
    for (i in seq_along(x$phenoSum)) {
      nTraits <- ncol(x$phenoSum[[i]])
      ## Print max 5 trait names.
      if (nTraits < 6) {
        traitNames <- colnames(x$phenoSum[[i]])
      } else {
        traitNames <- c(colnames(x$phenoSum[[i]])[1:4], "...",
                        tail(colnames(x$phenoSum[[i]]), 1))
      }
      cat("\tNumber of traits:", nTraits, "\n")
      cat("\tTraitnames:", paste(traitNames, collapse = ", ") , "\n")
      cat("\tNumber of genotypes:",
          attr(x = x$phenoSum[[i]], which = "nGeno"), "\n\n")
    }
  }
  if (!is.null(x$covarSum)) {
    cat("crosses")
    print(x$covarSum)
  }
}

#' Plot function for the class \code{gDataMPP}
#'
#' Creates a plot of an object of S3 class \code{gDataMPP}. The following types
#' of plot can be made:
#' \describe{
#' \item{\code{genMap}}{ A plot of the genetic map.}
#' \item{\code{allGeno}}{ A plot showing for all genotypes the IBD
#' probabilities of the parent with the highest probability per marker.}
#' \item{\code{singleGeno}}{ A plot for a single genotype showing the IBD
#' probabilities for all parents across the genome.}
#' \item{\code{pedigree}}{ A plot showing the structure of the pedigree of
#' the population.}
#' }
#' See the respective sections for more details on the plots.
#'
#' @section genMap:
#' A plot is made showing the lengths of the chromosomes and the position of
#' the markers that are present in the map. It is possible to highlight one
#' or more markers using the extra parameter \code{highlight}.
#'
#' @section allGeno:
#' A plot is made showing all genotypes and markers. Each combination of
#' genotype and marker is colored according to the parent with the highest
#' probability. A darker color indicates a higher probability.
#'
#' @section singleGeno:
#' A plot is made for a single genotype, specified by
#' \code{genotpye = "name_of_genotype"} showing the IBD probabilities for the
#' selected genotype for all parents across the genome.
#'
#' @section pedigree:
#' A plot is made showing the structure of the pedigree for the population in
#' the \code{gDataMPP} object.
#'
#' @param x An object of class \code{gDataMPP}.
#' @param ... Further arguments to be passed on to the actual plotting
#' functions.
#' @param plotType A character string indicating the type of plot to be made.
#' One of "genMap", "singleGeno", "allGeno" or "pedigree".
#' @param genotype A character string indicating the genotype for which the
#' plot should be made. Only for \code{plotType = "singleGeno"}.
#' @param title A character string, the title of the plot.
#' @param output Should the plot be output to the current device? If
#' \code{FALSE}, only a ggplot object is invisibly returned.
#'
#' @examples
#' ## Read phenotypic data.
#' pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
#'                                package = "statgenMPP"))
#' ## Rename first column to genotype.
#' colnames(pheno)[1] <- "genotype"
#'
#'
#' ## Compute IBD probabilities for simulated population - AxB, AxC.
#' ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
#'                   markerFiles = c(system.file("extdata/multipop", "AxB.txt",
#'                                               package = "statgenMPP"),
#'                                   system.file("extdata/multipop", "AxC.txt",
#'                                               package = "statgenMPP")),
#'                   pheno = pheno,
#'                   popType = "F4DH",
#'                   mapFile = system.file("extdata/multipop", "mapfile.txt",
#'                                         package = "statgenMPP"),
#'                   evalDist = 5)
#'
#' ## Plot the genetic map.
#' plot(ABC, plotType = "genMap")
#'
#' ## Plot the genetic map and highlight marker EXT_3_30.
#' plot(ABC, plotType = "genMap", highlight = "EXT_3_30")
#'
#' ## Plot the IBD probabilities across the genome for all genotypes.
#' plot(ABC, plotType = "allGeno")
#'
#' ## Plot the IBD probabilities for genotype AxB0001.
#' plot(ABC, plotType = "singleGeno", genotype = "AxB0001")
#'
#' ## Plot the pedigree.
#' plot(ABC, plotType = "pedigree")
#'
#' @returns A ggplot object is invisibly returned.
#'
#' @importFrom utils packageVersion
#' @export
plot.gDataMPP <- function(x,
                          ...,
                          plotType = c("genMap", "allGeno", "singleGeno",
                                       "pedigree"),
                          genotype = NULL,
                          title = NULL,
                          output = TRUE) {
  plotType <- match.arg(plotType)
  dotArgs <- list(...)
  map <- x$map
  markers <- x$markers
  parents <- dimnames(markers)[[3]]
  pedigree <- attr(x = x, which = "pedigree")
  genoCross <- attr(x = x, which = "genoCross")
  popType <- attr(x = x, which = "popType")
  if (plotType == "genMap") {
    highlight <- dotArgs$highlight
    missHighlight <- highlight[!highlight %in% rownames(map)]
    if (length(missHighlight) > 0) {
      stop("The following highlight markers are not in map:\n",
           paste(missHighlight, collapse = ", "))
    }
    if (length(highlight) > 0) {
      highlightDat <- map[highlight, ]
    } else {
      highlightDat <- NULL
    }
    p <- geneticMapPlot(map = map, highlight = highlightDat,
                        title = title, output = FALSE)
  } else if (plotType == "allGeno") {
    p <- allGenoPlot(markers = markers, map = map, parents = parents,
                     title = title)
  } else if (plotType == "singleGeno") {
    markers <- markers[genotype, , , drop = FALSE]
    p <- singleGenoPlot(markers = markers, map = map, parents = parents,
                        genotype = genotype, title = title)
  } else if (plotType == "pedigree") {
    p <- pedPlot(pedigree = pedigree, offSpring = rownames(markers),
                 popType = popType, genoCross = genoCross, title = title)
  }
  if (output) {
    plot(p)
  }
  invisible(p)
}

#' Multi round genome scans for QTL detection
#'
#' Multi round genome scans for QTL detection.\cr\cr
#' Several rounds of QTL detection are performed. First a model is fitted
#' without cofactors. If for at least one marker the \eqn{-log10(p)} value is
#' above the threshold the marker with the lowest p-Value is added as cofactor
#' in the next round of QTL detection. This process continues until there are
#' no new markers with a \eqn{-log10(p)} value above the threshold or until
#' the maximum number of cofactors is reached.
#'
#' By default only family specific effects and residual variances and no
#' kinship relations are included in the model. It is possible to include
#' kinship relations by either specifying \code{computeKin = TRUE}. When doing
#' so the kinship matrix is computed by averaging \eqn{Z Z^t} over all markers,
#' where \eqn{Z} is the genotype x parents matrix for the marker. It is also
#' possible to specify a list of precomputed chromosome
#' specific kinship matrices in \code{K}. Note that adding a kinship matrix
#' to the model increases the computation time a lot, especially for large
#' populations.
#'
#' @param MPPobj An object of class gDataMPP, typically the output of either
#' \code{\link{calcIBDMPP}} or \code{\link{readRABBITMPP}}.
#' @param trait A character string indicating the trait QTL mapping is done for.
#' @param QTLwindow A numerical value indicating the window around a QTL that
#' is considered as part of that QTL.
#' @param threshold A numerical value indicating the threshold for the
#' \eqn{-log10p} value of a marker to be considered a QTL.
#' @param maxCofactors A numerical value, the maximum number of cofactors to
#' include in the model. If \code{NULL} cofactors are added until no new
#' cofactors are found.
#' @param K A list of chromosome specific kinship matrices. If
#' \code{NULL} and \code{computeKin = FALSE} no kinship matrix is included in
#' the models.
#' @param computeKin Should chromosome specific kinship matrices be computed?
#' @param parallel Should the computation of variance components be done in
#' parallel? This requires a parallel back-end to be registered. See examples.
#' @param verbose Should progress and intermediate plots be output?
#'
#' @returns An object of class \code{QTLMPP}
#'
#' @examples
#' ## Read phenotypic data.
#' pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
#'                                package = "statgenMPP"))
#' ## Rename first column to genotype.
#' colnames(pheno)[1] <- "genotype"
#'
#'
#' ## Compute IBD probabilities for simulated population - AxB, AxC.
#' ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
#'                   markerFiles = c(system.file("extdata/multipop", "AxB.txt",
#'                                               package = "statgenMPP"),
#'                                   system.file("extdata/multipop", "AxC.txt",
#'                                               package = "statgenMPP")),
#'                   pheno = pheno,
#'                   popType = "F4DH",
#'                   mapFile = system.file("extdata/multipop", "mapfile.txt",
#'                                         package = "statgenMPP"),
#'                   evalDist = 5)
#'
#' ## Single-QTL Mapping.
#' ABC_SQM <- selQTLMPP(ABC, trait = "yield", maxCofactors = 0)
#'
#' ## Multi-QTL Mapping.
#' \dontrun{
#' ## Register parallel back-end with 2 cores.
#' doParallel::registerDoParallel(cores = 2)
#'
#' ## Run multi-QTL mapping.
#' ABC_MQM <- selQTLMPP(ABC, trait = "yield", parallel = TRUE)
#'
#' ## Run multi-QTL mapping - include kinship matrix.
#' ABC_MQM_kin <- selQTLMPP(ABC, trait = "yield", parallel = TRUE,
#'                         computeKin = TRUE)
#' }
#'
#' @seealso \code{\link{kinshipIBD}}
#'
#' @importFrom utils head tail
#' @export
selQTLMPP <- function(MPPobj,
                      trait = NULL,
                      QTLwindow = 10,
                      threshold = 3,
                      maxCofactors = NULL,
                      K = NULL,
                      computeKin = FALSE,
                      parallel = FALSE,
                      verbose = FALSE) {
  if (!inherits(MPPobj, "gDataMPP")) {
    stop("MPPobj should be an object of class gDataMPP.\n")
  }
  map <- MPPobj$map
  markers <- MPPobj$markers
  pheno <- MPPobj$pheno[[1]]
  covar <- MPPobj$covar
  mapOrig <- attr(x = MPPobj, which = "mapOrig")
  if (is.null(map)) {
    stop("MPP object should contain a map.\n")
  }
  if (is.null(markers)) {
    stop("MPP object should contain a marker matrix.\n")
  }
  if (length(dim(markers)) != 3) {
    stop("markers should be a 3D array with IBD probabilities.\n")
  }
  if (is.null(pheno)) {
    stop("MPP object should contain phenotypic data.\n")
  }
  if (!is.character(trait) || length(trait) > 1 ||
      !hasName(x = pheno, name = trait)) {
    stop("trait should be a character string of length one present in pheno.\n")
  }
  if (!is.numeric(QTLwindow) || length(QTLwindow) > 1 || QTLwindow < 0) {
    stop("QTLwindow should be a positive numerical value.\n")
  }
  if (!is.numeric(threshold) || length(threshold) > 1 || threshold < 0) {
    stop("threshold should be a positive numerical value.\n")
  }
  if (!is.null(maxCofactors) &&
      (!is.numeric(maxCofactors) || length(maxCofactors) > 1 ||
       maxCofactors < 0)) {
    stop("maxCofactors should be a positive numerical value.\n")
  }
  if (!is.null(K) &&
      (!is.list(K) || !all(sapply(K, FUN = function(k) {
        is.matrix(k) || inherits(k, "Matrix")})) ||
       length(K) != length(unique(map$chr)))) {
    stop("K should be a list of matrices of length equal to the ",
         "number of chromosomes in the map.\n")
  }
  if (is.null(maxCofactors)) {
    maxCofactors <- dim(markers)[2]
  }
  parents <- dimnames(markers)[[3]]
  nPar <- length(parents)
  markerNames <- colnames(markers)
  ## Restrict map to markers in markers.
  map <- map[rownames(map) %in% markerNames, ]
  ## Construct model data by merging phenotypic and genotypic data.
  ## Merge phenotypic data and covar (cross).
  if (!is.null(covar) && hasName(x = covar, name = "cross")) {
    modDat <- merge(pheno, covar, by.x = "genotype", by.y = "row.names")
  } else {
    modDat <- pheno
    modDat[["cross"]] <- factor(1)
  }
  ## Remove missing values for trait from modDat.
  ## Not strictly necessary, but it prevents warnings from LMMsolve later on.
  modDat <- droplevels(modDat[!is.na(modDat[[trait]]), ])
  ## Restrict markers to genotypes in modDat.
  markers <- markers[rownames(markers) %in% modDat[["genotype"]], , ]
  ## Restrict modDat to genotypes in markers.
  modDat <- droplevels(modDat[modDat[["genotype"]] %in% rownames(markers), ])
  ## Reorder markers according to genotypes in modDat.
  markers <- markers[modDat[["genotype"]], , ]
  ## Compute kinship matrices.
  if (computeKin) {
    K <- kinshipIBD(map = map, markers = markers)
  } else if (!is.null(K)) {
    K <- sapply(K, FUN = function(KChr) {
        ## Remove genotypes not in data.
        KChr[rownames(KChr) %in% modDat[["genotype"]],
             rownames(KChr) %in% modDat[["genotype"]]]
        ## Assure order of genotypes is identical to that in markers.
        KChr[order(match(rownames(KChr), rownames(markers))),
             order(match(colnames(KChr), rownames(markers)))]
    }, simplify = FALSE)
  } else {
    K <- NULL
  }
  if (!is.null(K)) {
    ## Compute spectral decomposition.
    Usc <- sapply(K, FUN = function(KChr) {
      eigKChr <- eigen(KChr)
      selEigVals <- eigKChr$values > sqrt(.Machine$double.eps)
      U <- eigKChr$vectors[, selEigVals]
      d <- eigKChr$values[selEigVals]
      U %*% diag(sqrt(d))
    }, simplify = FALSE)
  } else {
    Usc <- NULL
  }
  ## Initialize parameters.
  cofactors <- NULL
  mapScan <- map
  while (length(cofactors) < maxCofactors) {
    if (verbose) {
      cat(paste0("QTL scan for trait ", trait, ", ",
                 length(cofactors), " cofactors\n"))
    }
    scanRes <- scanQTL(modDat = modDat,
                       map = mapScan,
                       markers = markers,
                       parents = parents,
                       trait = trait,
                       QTLwindow = QTLwindow,
                       cof = cofactors,
                       Usc = Usc,
                       parallel = parallel,
                       verbose = verbose)
    if (verbose) {
      plotIntermediateScan(scanRes,
                           threshold = threshold,
                           cofactors = cofactors,
                           trait = trait)
    }
    ## Restrict to markers outside 'known' QTLRegions.
    scanSel <- scanRes[!scanRes[["QTLRegion"]] &
                         !is.na(scanRes[["minlog10p"]]), ]
    minlog10pMax <- max(scanSel[["minlog10p"]])
    if (minlog10pMax < threshold) {
      cofactors <- sort(cofactors)
      break
    }
    ## Add new cofactor to list of cofactors for next round of scanning.
    cofactors <- c(cofactors, scanSel[which.max(scanSel[["minlog10p"]]), "snp"])
    qtlWinScan <- scanRes[scanRes[["QTLRegion"]] &
                            !scanRes[["snp"]] %in% cofactors, "snp"]
    if (length(cofactors) == 1) {
      lowPScan <- scanRes[!is.na(scanRes[["minlog10p"]]) &
                            scanRes[["minlog10p"]] < 0.31, "snp"]
    }
    mapScan <- mapScan[!rownames(mapScan) %in% c(lowPScan, qtlWinScan), ]
  }
  ## Final run with all markers and all cofactors.
  scanRes <- scanQTL(modDat = modDat,
                     map = map,
                     markers = markers,
                     parents = parents,
                     trait = trait,
                     QTLwindow = QTLwindow,
                     cof = cofactors,
                     Usc = Usc,
                     se = TRUE,
                     parallel = parallel,
                     verbose = verbose)
  ## Flatten cofactor markers to 2D structure.
  if (!is.null(cofactors)) {
    markersCof <- do.call(cbind, lapply(X = cofactors, FUN = function(mrk) {
      markers[, mrk, ]
    }))
    colnames(markersCof) <- paste0(rep(cofactors, each = nPar), "_", parents)
    ## Merge markers to modDat.
    modDat <- merge(modDat, markersCof, by.x = "genotype", by.y = "row.names")
  }
  ## Fit model with all cofactors for computation of variance explained.
  finMod <- randomQTLmodel(modDat = modDat, map = map, parents = parents,
                           trait = trait, scanMrk = NULL, cofMrk = cofactors,
                           NULLmodel = TRUE)
  ## Get weighted residual error.
  crossN <- table(modDat[["cross"]])
  crossResErr <- tail(finMod$VarDf, length(crossN))
  resErr <- sum((crossN * crossResErr[["Variance"]])) / sum(crossN)
  ## Construct data.frame with explained variances.
  varQTL <- head(finMod$VarDf, length(cofactors))
  varAllQTLs <- sum(varQTL[["Variance"]])
  varQTL[["varExpl"]] <- varQTL[["Variance"]] / (varAllQTLs + resErr)
  varQTL <- varQTL[c("VarComp", "varExpl")]
  ## Construct GWAResult and signSnp
  colnames(scanRes)[colnames(scanRes) == "minlog10p"] <- "LOD"
  GWARes <- scanRes[ , colnames(scanRes) != "QTLRegion"]
  signSnp <- scanRes[scanRes[["QTLRegion"]], colnames(scanRes) != "QTLRegion"]
  ## Add explained variance.
  signSnp <- merge(signSnp, varQTL, by.x = "snp", by.y = "VarComp",
                   all.x = TRUE, sort = FALSE)
  ## Add nearest real marker.
  signSnp[["mrkNear"]] <- sapply(X = seq_len(nrow(signSnp)), FUN = function(i) {
    mrkChr <- signSnp[i, "chr"]
    mrkPos <- signSnp[i, "pos"]
    mapOrigChr <- mapOrig[mapOrig[["chr"]] == mrkChr, ]
    mrkNear <- mapOrigChr[which.min(abs(mapOrigChr[["pos"]] - mrkPos)), ]
    return(rownames(mrkNear))
  })
  signSnp[["snpStatus"]] <-
    as.factor(ifelse(signSnp[["snp"]] %in% cofactors,
                     "significant SNP",
                     "within QTL window of significant SNP"))
  ## Construct GWASInfo
  GWASInfo <- list(parents = parents)
  res <- createGWAS(GWAResult = list(pheno = GWARes),
                    signSnp = list(pheno = signSnp),
                    kin = K,
                    thr = list(pheno = setNames(threshold, trait)),
                    GWASInfo = GWASInfo)
  ## Add QTLMPP class to simplify providing generic functions.
  class(res) <- c("QTLMPP", class(res))
  return(res)
}

#' Read IBD probabilities
#'
#' Read a file with IBD probabilities computed by the RABBIT software package.
#' It is possible to additionally read the pedigree file that is also used by
#' RABBIT. Reading this file allows for plotting the pedigree. Phenotypic data
#' can be added from a data.frame.
#'
#' @param infile A character string, a link to a .csv file with IBD
#' probabilities.
#' @param pedFile A character string, a link to a .csv file with pedigree
#' information as used by RABBIT as input.
#' @param pheno A data frame with at least columns "genotype" for the
#' "genotype" and one or more numerical columns containing phenotypic
#' information. A column "cross" can be used for indicating the cross the
#' genotype comes from.
#'
#' @returns An object of class \code{gDataMPP} with map and markers
#' corresponding to the imported information in the imported .csv file.
#'
#' @examples
#' \dontrun{
#' ## Read RABBIT data for barley.
#' genoFile <- system.file("extdata/barley", "barley_magicReconstruct.zip",
#'                        package = "statgenMPP")
#' barleyMPMPP <- readRABBITMPP(unzip(genoFile, exdir = tempdir()))
#' }
#'
#' @references Zheng, Chaozhi, Martin P Boer, and Fred A Van Eeuwijk.
#' “Recursive Algorithms for Modeling Genomic Ancestral Origins in a Fixed
#' Pedigree.” G3 Genes|Genomes|Genetics 8 (10): 3231–45.
#' https://doi.org/10.1534/G3.118.200340.
#'
#' @importFrom utils hasName unzip
#' @export
readRABBITMPP <- function(infile,
                          pedFile = NULL,
                          pheno = NULL) {
  if (!is.null(pheno)) {
    if (!inherits(pheno, "data.frame")) {
      stop("pheno should be a data.frame.\n")
    }
    ## If pedFile is included cross will be read from there.
    minCols <- "genotype"
    missCols <- minCols[!sapply(X = minCols, FUN = function(minCol) {
      hasName(x = pheno, name = minCol)
    })]
    if (!is.null(pedFile) && hasName(x = pheno, name = "cross")) {
      pheno <- pheno[-which(colnames(pheno) == "cross")]
    }
    if (length(missCols) > 0) {
      stop("The following columns are missing in pheno:\n",
           paste(missCols, collapse = ", "))
    }
    trtCols <- colnames(pheno)[!colnames(pheno) %in% c(minCols, "cross")]
    nonNumCols <- trtCols[!sapply(X = pheno[trtCols], FUN = is.numeric)]
    if (length(nonNumCols) > 0) {
      stop("The following columns in pheno are not numeric:\n",
           paste(nonNumCols, collapse = ", "))
    }
  }
  rabbitRes <- statgenIBD::readRABBIT(infile, pedFile)
  covar <- NULL
  genoCross <- attr(x = rabbitRes, which = "genoCross")
  if (!is.null(pheno) && hasName(x = pheno, name = "cross")) {
    ## Split pheno in pheno and covar.
    covar <- pheno["cross"]
    rownames(covar) <- pheno[["genotype"]]
    pheno <- pheno[-which(colnames(pheno) == "cross")]
    genoCross <- covar
    genoCross[["geno"]] <- rownames(covar)
  } else if (!is.null(genoCross)) {
    covar <- genoCross
    rownames(covar) <- covar[["geno"]]
    covar[["geno"]] <- NULL
  }
  ## Create gDataMPP object.
  res <- createGDataMPPInternal(geno = rabbitRes$markers,
                                map = rabbitRes$map,
                                pheno = pheno,
                                covar = covar)
  attr(x = res, which = "popType") <- rabbitRes$popType
  attr(x = res, which = "genoCross") <- genoCross
  attr(x = res, which = "mapOrig") <- rabbitRes$map
  return(res)
}

### Test readRABBITMPP

## Define input files.

genoFile <- system.file("extdata/barley", "barley_magicReconstruct.zip",
                        package = "statgenMPP")
pedFile <- system.file("extdata/barley", "barley_pedInfo.csv",
                       package = "statgenMPP")

## Checks for correct input.
expect_error(readRABBITMPP(infile = 1),
             "infile should be a character string indicating a readable")
expect_error(readRABBITMPP(infile = "tst"),
             "infile should be a character string indicating a readable")
expect_error(readRABBITMPP(infile = "tst.csv"),
             "infile should be a character string indicating a readable")

expect_error(readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                           pedFile = 1),
             "pedFile should be a character string indicating a readable")
expect_error(readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                           pedFile = "tst"),
             "pedFile should be a character string indicating a readable")
expect_error(readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                           pedFile = "tst.csv"),
             "pedFile should be a character string indicating a readable")

expect_error(readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                           pheno  = 1),
             "pheno should be a data.frame")
expect_error(readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                           pheno  = barleyPheno["cross"]),
             "The following columns are missing in pheno")
barleyPhenoChr <- barleyPheno
barleyPhenoChr[["Awn_length"]] <- as.character(barleyPhenoChr[["Awn_length"]])
expect_error(readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                           pheno  = barleyPhenoChr),
             "The following columns in pheno are not numeric")

## Different combinations of inputs should give similar output.
expect_silent(barleyMPP <-
                readRABBITMPP(infile = unzip(genoFile, exdir = tempdir())))
expect_silent(barleyMPP2 <-
                readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                              pedFile = pedFile))
expect_silent(barleyMPP3 <-
                readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                              pheno = barleyPheno))
expect_silent(barleyMPP4 <-
                readRABBITMPP(infile = unzip(genoFile, exdir = tempdir()),
                              pedFile = pedFile, pheno = barleyPheno))

## General structure.
expect_inherits(barleyMPP4, "gDataMPP")

expect_inherits(barleyMPP4$map, "data.frame")
expect_equal(dim(barleyMPP4$map), c(355, 2))

expect_inherits(barleyMPP4$markers, "array")
expect_equal(dim(barleyMPP4$markers), c(916, 355, 5))

expect_inherits(barleyMPP4$pheno$pheno, "data.frame")

expect_inherits(barleyMPP4$covar, "data.frame")

expect_inherits(attr(barleyMPP2, "genoCross"), "data.frame")
expect_null(attr(barleyMPP2, "pedigree"))

## map and markers should be the same for all.
expect_equal(barleyMPP$map, barleyMPP4$map)
expect_equal(barleyMPP$markers, barleyMPP4$markers)

## covar read from pedFile.
expect_equal(barleyMPP2$covar, barleyMPP4$covar)
expect_equal(attr(barleyMPP2, "pedigree"), attr(barleyMPP4, "pedigree"))
expect_equal(attr(barleyMPP2, "genoCross"), attr(barleyMPP4, "genoCross"))

## Covar read from pheno should be identical to that read from pedFile.
expect_equal(barleyMPP3$pheno, barleyMPP4$pheno)
expect_equal(barleyMPP3$covar, barleyMPP4$covar)

#' Phenotypic data for awn length in barley
#'
#' @format A data.frame with 916 rows and 3 columns.
#' \describe{
#' \item{genotype}{Genotype}
#' \item{Awn_length}{Awn_length in mm.}
#' \item{cross}{The cross the genotype belongs to.}
#' }
#' @references Fine mapping of a major QTL for awn length in barley using a
#' multiparent mapping population. Liller CB, Walla A, Boer MP, Hedley P,
#' Macaulay M, Effgen S, von Korff M, van Esse GW, Koornneef M.
#' Theor Appl Genet. 2017 Feb;130(2):269-281. \doi{10.1007/s00122-016-2807-y}.
"barleyPheno"

#' Pre-computed SQM output maize
#'
#' Pre-computed SQM output for maize data used in vignette.
"maizeSQM"

#' Pre-computed MQM output maize
#'
#' Pre-computed MQM output for maize data used in vignette.
"maizeMQM"

#' Pre-computed MQM output barley
#'
#' Pre-computed MQM output for barley data used in vignette.
"barleyMQM"

#' one position fitted in the mixed model
#'
#' @importFrom stats as.formula
#' @keywords internal
randomQTLmodel <- function(modDat,
                           map,
                           parents,
                           trait = "yield",
                           scanMrk = NULL,
                           cofMrk = NULL,
                           NULLmodel = FALSE) {
  nCross <- length(unique(modDat[["cross"]]))
  nCof <- length(cofMrk)
  if (nCross == 1) { # for MAGIC-type pop with one cross.
    fixed <- as.formula(paste(trait, "~1"))
  } else { # for NAM or diallel-type pops with > 1 cross
    fixed <- as.formula(paste(trait, "~cross"))
  }
  selMrk <- c(cofMrk, if (!NULLmodel) scanMrk)
  Lgrp <- list()
  ## MB: add grp() terms to random part of model
  ranTerm <- NULL
  if (length(selMrk) > 0) {
    for (selName in selMrk) {
      selIBDNames <- paste0(selName, "_", parents)
      Lgrp[[selName]] <- which(colnames(modDat) %in% selIBDNames)
      if (is.null(ranTerm)) {
        ranTerm <- paste0(ranTerm, "~grp(`", selName, "`)")
      } else {
        ranTerm <- paste0(ranTerm, "+grp(`", selName, "`)")
      }
    }
  }
  if (!is.null(ranTerm)) {
    ranTerm = as.formula(ranTerm)
  }
  fitMod <- LMMsolver::LMMsolve(fixed = fixed,
                                random = ranTerm,
                                group = if (length(Lgrp) > 0) Lgrp,
                                residual = ~cross,
                                data = modDat,
                                tolerance = 1.0e-3)
  return(fitMod)
}

#' Create an object of class gDataMPP
#'
#' Function for creating an object of class gDataMPP from an object of class
#' IBDprob (computed or imported using statgenIBD) and phenotypic data.
#'
#' @param IBDprob An object of class \code{IBDprob}.
#' @param pheno A data frame with at least columns "genotype" for the
#' "genotype" and one or more numerical columns containing phenotypic
#' information. A column "cross" can be used for indicating the cross the
#' genotype comes from. This column is ignored if the `IBDprob` has an
#' attribute genoCross containing this information (the default behaviour).
#'
#' @returns An object of class \code{gDataMPP}
#'
#' @examples
#' ## Read phenotypic data.
#' pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
#'                                package = "statgenMPP"))
#' ## Rename first column to genotype.
#' colnames(pheno)[1] <- "genotype"
#'
#' ## Compute IBD probabilities for simulated population using statgenIBD - AxB
#' AB <- statgenIBD::calcIBD(popType = "F4DH",
#'                          markerFile = system.file("extdata/multipop", "AxB.txt",
#'                                                  package = "statgenMPP"),
#'                          mapFile = system.file("extdata/multipop", "mapfile.txt",
#'                                               package = "statgenMPP"),
#'                          evalDist = 5)
#'
#' ## Create an object of class gDataMPP from IBD probabilities and
#' ## phenotypic data.
#' ABMPP <- createGDataMPP(IBDprob = AB, pheno = pheno)
#'
#' @importFrom utils packageVersion
#' @export
createGDataMPP <- function(IBDprob,
                           pheno) {

  if (!inherits(IBDprob, "IBDprob")) {
    stop("IBDprob should be an object of class IBDprob.\n")
  }
  if (!inherits(pheno, "data.frame")) {
    stop("pheno should be a data.frame.\n")
  }
  genoCross <- attr(IBDprob, "genoCross")
  markers <- IBDprob$markers
  ## If pedFile is included cross will be read from there.
  minCols <- "genotype"
  missCols <- minCols[!sapply(X = minCols, FUN = function(minCol) {
    hasName(x = pheno, name = minCol)
  })]
  if (length(missCols) > 0) {
    stop("The following columns are missing in pheno:\n",
         paste(missCols, collapse = ", "))
  }
  if (!is.null(genoCross) && hasName(x = pheno, name = "cross")) {
    pheno <- pheno[-which(colnames(pheno) == "cross")]
  }
  trtCols <- colnames(pheno)[!colnames(pheno) %in% c(minCols, "cross")]
  nonNumCols <- trtCols[!sapply(X = pheno[trtCols], FUN = is.numeric)]
  if (length(nonNumCols) > 0) {
    stop("The following columns in pheno are not numeric:\n",
         paste(nonNumCols, collapse = ", "))
  }
  if (hasName(x = pheno, name = "cross")) {
    ## Split pheno in pheno and covar.
    covar <- pheno["cross"]
    rownames(covar) <- pheno[["genotype"]]
    pheno <- pheno[-which(colnames(pheno) == "cross")]
  } else {
    ## Construct covar from genoCross.
    covar <- genoCross["cross"]
    rownames(covar) <- genoCross[["geno"]]
  }
  ## Create gDataMPP object.
  res <- createGDataMPPInternal(geno = markers, map = IBDprob$map,
                                pheno = pheno, covar = covar)
  attr(x = res, which = "popType") <- IBDprob$popType
  attr(x = res, which = "genoCross") <- genoCross
  attr(x = res, which = "mapOrig") <- IBDprob$map
  attr(x = res, which = "pedigree") <- IBDprob$pedigree
  return(res)
}

### Test createGDataMPP

markerFile <- system.file("extdata/multipop", "AxB.txt",
                          package = "statgenMPP")
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")

pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                package = "statgenMPP"))
pheno2 <- pheno[, -1, drop = FALSE]
pheno3 <- pheno
pheno3[["yield"]] <- as.character(pheno3[["yield"]])


## Compute IBD probabilities using statgenIBD.
tstIBD <- statgenIBD::calcIBD(popType = "F4DH", markerFile = markerFile,
                              mapFile = mapFile, evalDist = 5)

expect_error(createGDataMPP(IBDprob = 1),
             "IBDprob should be an object of class IBDprob")
expect_error(createGDataMPP(IBDprob = tstIBD, pheno = 1),
             "pheno should be a data.frame")
expect_error(createGDataMPP(IBDprob = tstIBD, pheno = pheno2),
             "The following columns are missing in pheno")
expect_error(createGDataMPP(IBDprob = tstIBD, pheno = pheno3),
             "The following columns in pheno are not numeric")

tstMPP <- createGDataMPP(tstIBD, pheno)

expect_inherits(tstMPP, "gDataMPP")
expect_equal_to_reference(tstMPP, "gDataMPP")

## Check that cross is copied from pheno.
pheno4 <- pheno
pheno4[["cross"]] <- factor(1)

tstMPP_cross <- createGDataMPP(tstIBD, pheno4)
expect_inherits(tstMPP_cross$covar, "data.frame")
expect_null(tstMPP_cross$pheno$pheno[["cross"]])

#' @importFrom utils getFromNamespace
#' @keywords internal
manhattanPlot <- getFromNamespace(x = "manhattanPlot", ns = "statgenGWAS")

#' @keywords internal
geneticMapPlot <- getFromNamespace(x = "geneticMapPlot", ns = "statgenGWAS")

#' @keywords internal
createGWAS <- getFromNamespace(x = "createGWAS", ns = "statgenGWAS")

#' @keywords internal
#' @exportS3Method
plot.GWAS <- getFromNamespace(x = "plot.GWAS", ns = "statgenGWAS")

#' @keywords internal
nearestPD <- getFromNamespace(x = "nearestPD", ns = "statgenGWAS")

#' @keywords internal
markers3DtoMat <- getFromNamespace(x = "markers3DtoMat", ns = "statgenIBD")

#' @keywords internal
allGenoPlot <- getFromNamespace(x = "allGenoPlot", ns = "statgenIBD")

#' @keywords internal
singleGenoPlot <- getFromNamespace(x = "singleGenoPlot", ns = "statgenIBD")

#' @keywords internal
pedPlot <- getFromNamespace(x = "pedPlot", ns = "statgenIBD")

#' @keywords internal
constructRinv <- getFromNamespace(x = "constructRinv", ns = "LMMsolver")

#' @keywords internal
sparseMixedModels <- getFromNamespace(x = "sparseMixedModels", ns = "LMMsolver")

---
title: "QTL Mapping in Multi-Parent Populations"
author: "Wenhao Li, Martin Boer, and Bart-Jan van Rossum"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: false
    number_sections: false
bibliography: bibliography.bib
link-citations: yes
vignette: >
  %\VignetteIndexEntry{QTL Mapping in Multi-Parent Populations}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.dim = c(7, 4)
)
library(statgenMPP)
op <- options(width = 90, 
              digits = 3)
```

## The statgenMPP package

The `statgenMPP` package is developed as an easy-to-use package for QTL mapping in biparental and multi-parent populations. The package has many ways of visualizing inputs and results.

This vignette describes in detail how to perform the IBD calculations and how to do QTL mapping using the IBD probabilities in a linear mixed model framework, see @Li2021 and @Li2022 for details. The Linear Model Equations are solved using the [LMMsolver](https://biometris.github.io/LMMsolver/) R package [@boer2023]. This will be illustrated using three example data sets. The first data set is a simulated NAM data set with three parents, that is relatively small and runs fast and is mainly used to show the functionality of the package. Then two data sets from literature are used to show the validity of the results from the package, first a maize data set, the dent panel of the EU-NAM maize project (@Giraud2014). The second data set is a barley data set for awn length described in @Liller2017-jz.

## Example simulated data

As a first example for performing IBD calculations and QTL mapping for a multi-parent population we use a relatively simple simulated data set. The example contains simulated data for two F4DH populations. The population type F4DH is cross between two parents, followed by 3 generations of selfings, followed by a DH generation, see [statgenIBD](https://biometris.github.io/statgenIBD/) for details.

For the first population the parents where A and B, for the second the parents where A and C. The first population consists of 100 individuals, the second of 80 individuals. This is a simple example of a NAM population, having parent A as central parent. The data is simulated with three QTLs, on chromosome 1, 2, and 3. All necessary data for this population is available in the package.

Before doing QTL detection we compute IBD probabilities on a grid of positions along the genome. This can be done using the `calcIBDMPP` function in the `statgenMPP` package. To perform IBD calculations a marker file is required for each of the populations. These files should be a tab-delimited file with first column ID identifying the genotype. The following columns should contain marker information. The first rows should contain the parents used in the cross. As an example, the file for the first cross, AxB, starts like this:

```{r simMarkerDat, results='asis', echo=FALSE}
simMrkDat <- read.delim(system.file("extdata/multipop", "AxB.txt", package = "statgenMPP"))
knitr::kable(simMrkDat[1:4, 1:5])
```

In this example markers M1_1 and M1_4 are segregating in the AxB cross, M1_2 and M1_3 not.\
A map file is also required. This should also be a tab-delimited file with three columns, "marker name", "chromosome" and "position". The map file cannot contain a header and has to be identical for all crosses.

Phenotypic data can be added as a `data.frame` when computing IBD probabilities. Such a `data.frame` should have a first column "genotype" and all other columns have to be numerical. For the simulated NAM population the `data.frame` with phenotypic data starts like this:

```{r simPhenoDat, results='asis', echo=FALSE}
simPhenoDat <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt", package = "statgenMPP"))
knitr::kable(head(simPhenoDat))
```

The phenotypic data only contains a single trait, yield. When performing IBD calculations and specifying phenotypic data, the phenotypic data will be combined with computed IBD probabilities. For this, the genotype specified in the ID column in the marker file(s) will be matched with the genotype in the genotype column in the `data.frame` with phenotypic information. Phenotypic data for all crosses can either be added from a single `data.frame` containing phenotypic data for all crosses, or a `list` of `data.frames` each containing phenotypic data for a single cross.

```{r simIBD}
## Specify files containing markers.
# One file for each of the two crosses.
markerFiles <- c(system.file("extdata/multipop", "AxB.txt", 
                             package = "statgenMPP"),
                 system.file("extdata/multipop", "AxC.txt", 
                             package = "statgenMPP"))

## Specify file containing map.
# Both crosses use the same map file.
mapFile <- system.file("extdata/multipop", "mapfile.txt",
                       package = "statgenMPP")

## Read phenotypic data
phenoDat <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                                   package = "statgenMPP"))
# Check contents.
head(phenoDat)

## Perform IBD calculations. 
ABCMPP <- calcIBDMPP(crossNames = c("AxB", "AxC"),  
                     markerFiles = markerFiles,
                     pheno = phenoDat,
                     popType = "F4DH",
                     mapFile = mapFile,
                     evalDist = 5)
```

With `calcIBDMPP` IBD probabilities are computed on a grid for each of the crosses separately and then combined into a single output object. The value of `evalDist` can be used to specify the maximum distance (in cM) between two evaluation points on the grid. The exact distance depends on the length of the chromosomes. The output is stored in an object of class `gDataMPP` (genomic Data for Multi Parent Populations) in which information about map, markers, and phenotypic data is combined. With the `summary` function we can get some insight in this information.

```{r sumABCMPP}
## Print summary
summary(ABCMPP)
```

To get a further idea about the population and the computed IBD probabilities we can visualize the results. First we have a look at structure of the pedigree of the population using `plotType = "pedigree"` to get a general idea of what the design looks like.

```{r plotPABCMPP}
## Plot structure of the pedigree.
plot(ABCMPP, plotType = "pedigree")
```

Next we look at the genetic map using `plotType = "genMap"`. This will display the genetic map of the population showing the length of each of the chromosomes and indicating the positions where the IBD probabilities were calculated. Optionally it is possible to highlight one or more markers using `highlight` argument.

```{r plotGABCMPP}
## Plot genetic map.
# Highlight marker on chromosome 3 at position 40.
plot(ABCMPP, plotType = "genMap", highlight = "EXT_3_40")
```

Finally we visualize the computed IBD probabilities across the genome for a selected genotype using `plotType = "singleGeno"`. This plot will show the IBD probabilities for all parents for all positions on the genome for the selected genotype.

```{r plotSGABCMPP, fig.height=10}
## Plot IBD probabilities for genotype AxB0001.
plot(ABCMPP, plotType = "singleGeno", genotype = "AxB0001")
```

Using the computed IBD probabilities we can now do the actual QTL Mapping using the `selQTLMPP` function. First we perform a SQM by setting `maxCofactors = 0`. Our trait of interest, yield, is specified in the `trait` argument of the function.

```{r ABCSQM}
## Perform Single-QTL Mapping.
ABCSQM <- selQTLMPP(MPPobj = ABCMPP,
                    trait = "yield",
                    maxCofactors = 0)
```

The results of the SQM can be plotted using the `plot` function with `plotType = "QTLProfile"`.

```{r QPABCSQM}
## Plot QTL Profile for ABC SQM.
plot(ABCSQM, plotType = "QTLProfile")
```

Already from the SQM the position of the three simulated QTLs is quite clear. We can get an even better result using MQM. For that, again we use the `selQTLMPP` function. We don't specify the `maxCofactors` to let the algorithm determine the number of cofactors based on the threshold and QTLwindow. As long as new markers are found with a $-log10(p)$ value above `threshold` and the maximum number of cofactors is not reached, a new round of scanning is done. In the new round of scanning the marker with the highest $-log10(p)$ value from the previous round is added to the cofactors. Based on the profile plot for the SQM the `threshold` is set to 3. The `QTLwindow` is not specified and therefore left at its default value of 10cM.

```{r ABCMQM}
## Perform Multi-QTL Mapping.
ABCMQM <- selQTLMPP(MPPobj = ABCMPP,
                    trait = "yield",
                    threshold = 3)
```

It is possible to include a polygenic term in the model to control for background genetic information. To do this a list of chromosome specific kinship matrices can either be computed by the `selQTLMPP` function or specified by the user. In the former case it is enough to specify `computeKin = TRUE`. When doing so the kinship matrix is computed by averaging $Z Z^t$ over all markers, where $Z$ is the genotype $\times$ parents matrix with IBD probabilities for the marker. A list of precomputed chromosome specific kinship matrices can be specified in `K`. Note that adding a kinship matrix to the model increases the computation time a lot, especially for large populations. It is advisable to use parallel computing when doing so (see [Parallel computing](#parCom)).

```{r ABCMQM_kin, eval=FALSE}
## Perform Multi-QTL Mapping.
# Compute kinship matrices.
ABCMQM_kin <- selQTLMPP(MPPobj = ABCMPP,
                        trait = "yield",
                        threshold = 3,
                        computeKin = TRUE)
```

Next we can visualize the results. First we plot the positions of the QTLs found on the genetic map. This will produce a plot similar the the genetic map plot we have seen before, but now the QTLs will be highlighted. This plot can be made by specifying `plotType = "QTLRegion"`

```{r plotQRABCMQM}
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "QTLRegion")
```

As for the SQM we can also plot the QTL profile. The QTLs found will be highlighted in the profile in red.

```{r plotQPABCMQM}
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "QTLProfile")
```

It is also possible to plot the size of the parental effects for each of the QTLs found. Positive effects of a parent on the trait will be indicated by shades of red, negative effects by shades of blue. The stronger the color, the larger the effect for the specific parent is. This plot can be made using `plotType = "parEffs"`.

```{r plotPEABCMQM}
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "parEffs")
```

Also a combined plot of the QTL profile and the parental can be made. In this plot the two previous plots are plotted above each other with the chromosomes and positions aligned to allow for easily getting an overview of which effect belongs to which QTL in the QTL profile. This plot can be made using `plotType = "QTLProfileExt"`.

```{r plotQPEABCMQM}
## Plot QTL Profile for ABC MQM.
plot(ABCMQM, plotType = "QTLProfileExt")
```

Finally a plot of the confidence intervals around the estimated parental effects can be made for each of the QTLs found. This plot can be made using `plotType = "parCIs"`. The plot will show per QTL the effects for each of the parents, as well as the confidence intervals around those effects (calculated as $2 \times se$). Confidence intervals that don't contain the origin, are assumed to be significant and are shown in red for positive effects and in blue for negative effects. All other intervals are shown in black.

```{r plotCIABCMQM}
## Plot confidence intervals for parental effects for ABC MQM.
plot(ABCMQM, plotType = "parCIs")
```

A summary of the QTL-analysis gives a short overview containing the total number of markers and the number of QTLs found. Also for all QTL their position on the chromosome is shown as well as the nearest marker on the original map, the explained variance and the effects and the standard errors of all parents.

```{r sumABCMQM}
## Print summary
summary(ABCMQM)
```

From the output of `selQTLMPP` the p-Values and effects for all markers can be extracted. They are stored in a `data.table` within the output object. The example below shows how to extract them. The output will contain columns evalPos, chr and pos with name, chromosome number and evaluation position and columns pValue and eff_par1, eff_par2, eff_par.. with the effects of all parents for that marker.

```{r extractABCRes}
## Extract results of QTL mapping.
ABCMQMres <- ABCMQM$GWAResult$yield
head(ABCMQMres[, 1:8])
```

It is also possible to only extract the markers that are either QTLs or within the window of one of the selected QTLs, as specified when calling the `selQTLMPP` function. As for the full results, this information in stored in the output as a `data.table` that can be extracted as shown below. The columns in this `data.table` are identical to those in the full results except for an additional column at the end, snpStatus, that shows whether a marker is a QTL or within the window of a QTL.

```{r extractABCQTL}
## Extract QTLs and markers within QTL windows.
ABCMQMQTL <- ABCMQM$signSnp$yield
head(ABCMQMQTL[, c(2:8, 10)])
```

## Example maize

As a second example we use data from a maize NAM population described in @Giraud2014. The NAM population consists of 10 bi-parental doubled haploid (DH) crosses with central parent F353. The total population consists of 841 individuals. Several traits were measured in four locations across Europe. We calculated the best linear unbiased estimations (BLUEs) of those traits using the R package [statgenSTA](https://biometris.github.io/statgenSTA/). As an example, we perform QTL mapping for only the mean value of the number of days to silking ("mean_DtSILK") across all locations. The data for this population is available from the package in zipped format.

As for the simulated data, before doing QTL detection we first compute IBD probabilities on a grid of positions along the genome.

```{r maizeIBD}
## Define names of crosses.
crosses <- paste0("F353x", c("B73", "D06", "D09", "EC169", "F252", "F618", 
                             "Mo17", "UH250", "UH304", "W117"))
head(crosses)

## Specify files containing crosses.
## Extract them in a temporary directory.
tempDir <- tempdir()
crossFiles <- unzip(system.file("extdata/maize/maize.zip", package = "statgenMPP"), 
                    files = paste0(crosses, ".txt"), exdir = tempDir)

## Specify file containing map.
mapFile <- unzip(system.file("extdata/maize/maize.zip", package = "statgenMPP"), 
                 files = "map.txt", exdir = tempDir)

## Read phenotypic data.
phenoFile <- unzip(system.file("extdata/maize/maize.zip", package = "statgenMPP"),
                   files = "EUmaizePheno.txt", exdir = tempDir)
phenoDat <- read.delim(phenoFile)
head(phenoDat[, 1:5])

## Perform IBD calculations. 
maizeMPP <- calcIBDMPP(crossNames = crosses, 
                       markerFiles = crossFiles,
                       pheno = phenoDat,
                       popType = "DH",
                       mapFile = mapFile,
                       evalDist = 5)
```

We then have a look at the summary and some of the plots to get an overview of the pedigree and the computed probabilities.

```{r sumMaizeIBD}
## Print summary
summary(maizeMPP)
```

```{r plotPmaizeIBD}
## Plot structure of the pedigree.
plot(maizeMPP, plotType = "pedigree")
```

Now we can use the computed IBD probabilities to perform SQM.

```{r maizeSQM, eval=FALSE}
## Perform Single-QTL Mapping.
maizeSQM <- selQTLMPP(MPPobj = maizeMPP,
                      trait = "mean_DtSILK",
                      maxCofactors = 0)
```

We plot the QTL profile for the SQM to get an idea of reasonable values to use for the `threshold` and `QTLwindow` in the MQM that we do next.

```{r QPmaizeSQM}
## Plot QTL Profile for maize SQM.
plot(maizeSQM, plotType = "QTLProfile")
```

Based on the plot for the SQM the `threshold` is set to 5 to restrict a bit the number of QTLs that will be detected in the MQM. The `QTLwindow` is not specified and therefore left at its default value of 10cM.

```{r maizeMQM, eval=FALSE}
## Perform Multi-QTL Mapping.
maizeMQM <- selQTLMPP(MPPobj = maizeMPP,
                      trait = "mean_DtSILK",
                      threshold = 5)
```

We only look at the combined plot of the QTL Profile and the parental effects. This should give us the most direct insight in the QTLs found and the effects the different parents in the crosses have.

```{r plotQPEmaizeMQM}
## Plot QTL Profile for maize MQM.
plot(maizeMQM, plotType = "QTLProfileExt")
```

Finally we can have a look at the confidence intervals around the parental effects for the QTLs found.

```{r plotCIamizeMQM}
## Plot confidence intervals for parental effects for maize MQM.
plot(maizeMQM, plotType = "parCIs")
```

## Example barley

Instead of performing IBD calculations directly with the package, it is also possible to import IBD probabilities computed using RABBIT software [@Zheng2014; @Zheng2015; @Zheng2018]. The main advantage of using RABBIT for IBD calculations is that it can handle complex pedigree populations and therefore can also be used in cases where the population structure is more complex than those that can be computed using `statgenIBD`, e.g. in the maize NAM population described before.

As an example we use a barley population described in @Liller2017-jz. This MPP design consists of 5 parents. Four wild parents were crossed with the cultivar Morex and then backcrossed with Morex once. Individuals from the four families from the backcrosses were then crossed with each other as in a full diallel design, which generated six F6 families through five generations of selfing. The trait of interest for this population is awn length ("Awn_length"). As for the maize NAM population, the data for this population is available in zipped format in the package.

RABBIT output can be read using the `readRABBITMPP` function in `statgenMPP`. This has as input the standard RABBIT output summary file and the pedigree file that needs to be provided to RABBIT as well. This pedigree file is an optional input and is only used for plotting the pedigree structure of the population. Without it QTL mapping can still be performed. As for `calcIBDMPP` the phenotypic data has to be provided as a `data.frame`. This `data.frame` has been included in the package.

```{r barleyIBD}
## Specify files containing RABBIT output.
## Extract in a temporary directory.
tempDir <- tempdir()
inFile <- unzip(system.file("extdata/barley/barley_magicReconstruct.zip", 
                            package = "statgenMPP"), exdir = tempDir)

## Specify pedigree file.
pedFile <- system.file("extdata/barley/barley_pedInfo.csv",
                       package = "statgenMPP")

## Read phenotypic data.
data("barleyPheno")

## read RABBIT output. 
barleyMPP <- readRABBITMPP(infile = inFile,
                           pedFile = pedFile,
                           pheno = barleyPheno)
```

As for the maize example we can summarize and plot the imported data to get a first idea of its content.

```{r sumPbarleyIBD}
## Summary.
summary(barleyMPP)
```

Performing SQM and MQM for imported RABBIT output works in the same way as for IBD probabilities computed directly in the package. Since a full scan would take long we precomputed the results and included them in the package.

```{r barleyMQM, eval=FALSE}
## Perform Multi-QTL Mapping with threshold 4.
barleyMQM <- selQTLMPP(MPPobj = barleyMPP,
                       trait = "Awn_length",
                       threshold = 4)
```

There is a very large QTL on chromosome 7. To be able to more clearly distinguish the differences between the other QTLs they are plotted separately.

```{r QPbarleyMQM16}
## Plot QTL Profile for barley MQM - chromosome 1-6.
plot(barleyMQM, plotType = "QTLProfileExt", chr = 1:6)
```

```{r QPbarleyMQM7}
## Plot QTL Profile for barley MQM - chromosome 7.
plot(barleyMQM, plotType = "QTLProfileExt", chr = 7)
```

Finally we can have a look at the confidence intervals around the parental effects for the QTLs found.

```{r plotCIbarleyMQM}
## Plot confidence intervals for parental effects for maize MQM.
plot(barleyMQM, plotType = "parCIs")
```

The QTLs found are very similar in both position, size and effects as described in @Liller2017-jz. This can also be clearly seen by comparing the summary of the MQM with the table of effects in this paper.

```{r sumBarleyMQM}
## Summary.
summary(barleyMQM)
```

## Parallel computing {#parCom}

To improve performance, it is possible to use parallel computing to perform the QTL mapping. To do this, a parallel back-end has to be specified by the user, e.g. by using `registerDoParallel` from the `doParallel` package (see the example below). Besides, in the `selQTLMPP` function the argument `parallel` has to be set to `TRUE`. With these settings the computations are done in parallel per chromosome. Of course, this doesn't have an effect on the output. The MQM below gives exactly the same results as the non-parallel one described earlier in the vignette.

```{r ABCMQMPar, eval = FALSE}
## Register parallel back-end with 2 cores.
doParallel::registerDoParallel(cores = 2)

## Perform Multi-QTL Mapping.
ABCMQM_Par <- selQTLMPP(MPPobj = ABCMPP,
                        trait = "yield",
                        threshold = 3,
                        parallel = TRUE)
```

## Summary

The properties of using statgenMPP for QTL mapping in MPPs have been shown with several examples we demonstrated here. This easy-to-use R package, integrating the HMM method for IBD calculation and the mixed-model approach for QTL mapping, provides us with a general framework to estimate multi-allelic QTL effects in terms of parent origins.

```{r winddown, include = FALSE}
options(op)
```

------------------------------------------------------------------------

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createGDataMPPInternal.R
\name{plot.gDataMPP}
\alias{plot.gDataMPP}
\title{Plot function for the class \code{gDataMPP}}
\usage{
\method{plot}{gDataMPP}(
  x,
  ...,
  plotType = c("genMap", "allGeno", "singleGeno", "pedigree"),
  genotype = NULL,
  title = NULL,
  output = TRUE
)
}
\arguments{
\item{x}{An object of class \code{gDataMPP}.}

\item{...}{Further arguments to be passed on to the actual plotting
functions.}

\item{plotType}{A character string indicating the type of plot to be made.
One of "genMap", "singleGeno", "allGeno" or "pedigree".}

\item{genotype}{A character string indicating the genotype for which the
plot should be made. Only for \code{plotType = "singleGeno"}.}

\item{title}{A character string, the title of the plot.}

\item{output}{Should the plot be output to the current device? If
\code{FALSE}, only a ggplot object is invisibly returned.}
}
\value{
A ggplot object is invisibly returned.
}
\description{
Creates a plot of an object of S3 class \code{gDataMPP}. The following types
of plot can be made:
\describe{
\item{\code{genMap}}{ A plot of the genetic map.}
\item{\code{allGeno}}{ A plot showing for all genotypes the IBD
probabilities of the parent with the highest probability per marker.}
\item{\code{singleGeno}}{ A plot for a single genotype showing the IBD
probabilities for all parents across the genome.}
\item{\code{pedigree}}{ A plot showing the structure of the pedigree of
the population.}
}
See the respective sections for more details on the plots.
}
\section{genMap}{

A plot is made showing the lengths of the chromosomes and the position of
the markers that are present in the map. It is possible to highlight one
or more markers using the extra parameter \code{highlight}.
}

\section{allGeno}{

A plot is made showing all genotypes and markers. Each combination of
genotype and marker is colored according to the parent with the highest
probability. A darker color indicates a higher probability.
}

\section{singleGeno}{

A plot is made for a single genotype, specified by
\code{genotpye = "name_of_genotype"} showing the IBD probabilities for the
selected genotype for all parents across the genome.
}

\section{pedigree}{

A plot is made showing the structure of the pedigree for the population in
the \code{gDataMPP} object.
}

\examples{
## Read phenotypic data.
pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                               package = "statgenMPP"))
## Rename first column to genotype.
colnames(pheno)[1] <- "genotype"


## Compute IBD probabilities for simulated population - AxB, AxC.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = c(system.file("extdata/multipop", "AxB.txt",
                                              package = "statgenMPP"),
                                  system.file("extdata/multipop", "AxC.txt",
                                              package = "statgenMPP")),
                  pheno = pheno,
                  popType = "F4DH",
                  mapFile = system.file("extdata/multipop", "mapfile.txt",
                                        package = "statgenMPP"),
                  evalDist = 5)

## Plot the genetic map.
plot(ABC, plotType = "genMap")

## Plot the genetic map and highlight marker EXT_3_30.
plot(ABC, plotType = "genMap", highlight = "EXT_3_30")

## Plot the IBD probabilities across the genome for all genotypes.
plot(ABC, plotType = "allGeno")

## Plot the IBD probabilities for genotype AxB0001.
plot(ABC, plotType = "singleGeno", genotype = "AxB0001")

## Plot the pedigree.
plot(ABC, plotType = "pedigree")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{barleyPheno}
\alias{barleyPheno}
\title{Phenotypic data for awn length in barley}
\format{
A data.frame with 916 rows and 3 columns.
\describe{
\item{genotype}{Genotype}
\item{Awn_length}{Awn_length in mm.}
\item{cross}{The cross the genotype belongs to.}
}
}
\usage{
barleyPheno
}
\description{
Phenotypic data for awn length in barley
}
\references{
Fine mapping of a major QTL for awn length in barley using a
multiparent mapping population. Liller CB, Walla A, Boer MP, Hedley P,
Macaulay M, Effgen S, von Korff M, van Esse GW, Koornneef M.
Theor Appl Genet. 2017 Feb;130(2):269-281. \doi{10.1007/s00122-016-2807-y}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kinshipIBD.R
\name{kinshipIBD}
\alias{kinshipIBD}
\title{Compute kinship matrix for IBD probabilities}
\usage{
kinshipIBD(markers, map = NULL, chrSpecific = TRUE)
}
\arguments{
\item{markers}{An n x m x p array with IBD probabilities with genotypes in
the rows (n), markers in the columns (m), and parents in the 3rd dimension
(p).}

\item{map}{A data.frame with columns \code{chr} for chromosome and
\code{pos} for position. Positions should be in centimorgan (cM). They
should not be cumulative over the chromosomes. Other columns are ignored.
Marker names should be in the row names. These should match the marker names
in the input file. Only required if \code{chrSpecific = TRUE}.}

\item{chrSpecific}{Should chromosome specific kinship matrices be
computed?}
}
\value{
A kinship matrix or a list of chromosome specific kinship matrices.
}
\description{
Compute a kinship matrix or a list of chromosome specific kinship matrices
for a 3D array of IBD probabilities. The kinship matrix is computed by
averaging \eqn{Z Z^t} over all markers, where \eqn{Z} is the genotype x
parents matrix for the marker. If \code{chrSpecific = TRUE} chromosome
specific kinship matrices are computed for each chromosome based only on the
markers on all other chromosomes.
}
\examples{
## Read phenotypic data.
pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                               package = "statgenMPP"))
## Rename first column to genotype.
colnames(pheno)[1] <- "genotype"


## Compute IBD probabilities for simulated population - AxB, AxC.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = c(system.file("extdata/multipop", "AxB.txt",
                                              package = "statgenMPP"),
                                  system.file("extdata/multipop", "AxC.txt",
                                              package = "statgenMPP")),
                  pheno = pheno,
                  popType = "F4DH",
                  mapFile = system.file("extdata/multipop", "mapfile.txt",
                                        package = "statgenMPP"),
                  evalDist = 5)

## Compute chromosome specific kinship matrices.
KChrSpec <- kinshipIBD(markers = ABC$markers, map = ABC$map)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createGDataMPPInternal.R
\name{summary.gDataMPP}
\alias{summary.gDataMPP}
\title{Summary function for the class \code{gDataMPP}}
\usage{
\method{summary}{gDataMPP}(object, ..., trials = NULL)
}
\arguments{
\item{object}{An object of class \code{gDataMPP}.}

\item{...}{Not used.}

\item{trials}{A vector of trials to include in the summary. These can
be either numeric indices or character names of list items in \code{pheno}.
If \code{NULL}, all trials are included.}
}
\value{
A list with a most four components:
\describe{
\item{mapSum}{A list with number of markers and number of chromosomes in
the map.}
\item{markerSum}{A list with number of markers, number of genotypes and
the distribution of the values within the markers.}
\item{phenoSum}{A list of data.frames, one per trial with a summary of all
traits within the trial.}
\item{covarSum}{A list of data.frames, one per trial with a summary of all
covariates within the trial.}
}
All components are only present in the output if the corresponding content is
present in the \code{gDataMPP} object.
}
\description{
Gives a summary for an object of S3 class \code{gDataMPP}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statgenMPP.R
\docType{package}
\name{statgenMPP-package}
\alias{statgenMPP}
\alias{statgenMPP-package}
\title{statgenMPP: QTL Mapping for Multi Parent Populations}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

For Multi Parent Populations (MPP) Identity By Descend (IBD) probabilities are computed using Hidden Markov Models. These probabilities are then used in a mixed model approach for QTL Mapping as described in Li et al. (\doi{10.1007/s00122-021-03919-7}).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://biometris.github.io/statgenMPP/index.html}
  \item \url{https://github.com/Biometris/statgenMPP/}
  \item Report bugs at \url{https://github.com/Biometris/statgenMPP/issues}
}

}
\author{
\strong{Maintainer}: Bart-Jan van Rossum \email{bart-jan.vanrossum@wur.nl} (\href{https://orcid.org/0000-0002-8673-2514}{ORCID})

Authors:
\itemize{
  \item Bart-Jan van Rossum \email{bart-jan.vanrossum@wur.nl} (\href{https://orcid.org/0000-0002-8673-2514}{ORCID})
  \item Martin Boer \email{martin.boer@wur.nl} (\href{https://orcid.org/0000-0002-1879-4588}{ORCID})
  \item Wenhao Li (\href{https://orcid.org/0000-0001-5719-5775}{ORCID})
}

Other contributors:
\itemize{
  \item Fred van Eeuwijk (\href{https://orcid.org/0000-0003-3672-2921}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{calcDistance}
\alias{calcDistance}
\title{cofactor selection based on the QTL windowsize}
\usage{
calcDistance(map, m1, m2)
}
\description{
cofactor selection based on the QTL windowsize
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scanQTL.R
\name{scanQTL}
\alias{scanQTL}
\title{one-round genome scan with specified cofactor(s)}
\usage{
scanQTL(
  modDat,
  map,
  markers,
  parents,
  QTLwindow = 10,
  cof = NULL,
  Usc = NULL,
  trait = NULL,
  maxIter = 100,
  se = FALSE,
  parallel = FALSE,
  verbose = FALSE
)
}
\description{
one-round genome scan with specified cofactor(s)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effectPlot.R
\name{effectPlot}
\alias{effectPlot}
\title{Create a matrix plot}
\usage{
effectPlot(
  effectDat,
  signSnp,
  map,
  xLab = "Chromosomes",
  yLab = "Parents",
  chrBoundaries,
  title = NULL,
  trait = NULL,
  ...,
  output = TRUE
)
}
\description{
Create a matrix plot of effect sizes and directions per parent for
significant snps.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createQTLMPP.R
\name{summary.QTLMPP}
\alias{summary.QTLMPP}
\title{Summary function for the class \code{QTLMPP}}
\usage{
\method{summary}{QTLMPP}(object, ...)
}
\arguments{
\item{object}{An object of class \code{QTLMPP}.}

\item{...}{Not used.}
}
\description{
Gives a summary for an object of S3 class \code{QTLMPP}.
}
\examples{
\dontrun{
## Read phenotypic data.
pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                               package = "statgenMPP"))
## Rename first column to genotype.
colnames(pheno)[1] <- "genotype"

## Compute IBD probabilities for simulated population - AxB, AxC.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = c(system.file("extdata/multipop", "AxB.txt",
                                              package = "statgenMPP"),
                                  system.file("extdata/multipop", "AxC.txt",
                                              package = "statgenMPP")),
                  pheno = pheno,
                  popType = "F4DH",
                  mapFile = system.file("extdata/multipop", "mapfile.txt",
                                        package = "statgenMPP"),
                  evalDist = 5)

## Multi-QTL Mapping.
ABC_MQM <- selQTLMPP(ABC, trait = "yield")

## Print summary.
summary(ABC_MQM)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createQTLMPP.R
\name{print.summary.QTLMPP}
\alias{print.summary.QTLMPP}
\title{Print summary of object of class \code{summary.QTLMPP}}
\usage{
\method{print}{summary.QTLMPP}(x, ...)
}
\arguments{
\item{x}{An object of class \code{summary.QTLMPP}.}

\item{...}{Not used.}
}
\description{
Print summary of object of class \code{summary.QTLMPP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readRabbitMPP.R
\name{readRABBITMPP}
\alias{readRABBITMPP}
\title{Read IBD probabilities}
\usage{
readRABBITMPP(infile, pedFile = NULL, pheno = NULL)
}
\arguments{
\item{infile}{A character string, a link to a .csv file with IBD
probabilities.}

\item{pedFile}{A character string, a link to a .csv file with pedigree
information as used by RABBIT as input.}

\item{pheno}{A data frame with at least columns "genotype" for the
"genotype" and one or more numerical columns containing phenotypic
information. A column "cross" can be used for indicating the cross the
genotype comes from.}
}
\value{
An object of class \code{gDataMPP} with map and markers
corresponding to the imported information in the imported .csv file.
}
\description{
Read a file with IBD probabilities computed by the RABBIT software package.
It is possible to additionally read the pedigree file that is also used by
RABBIT. Reading this file allows for plotting the pedigree. Phenotypic data
can be added from a data.frame.
}
\examples{
\dontrun{
## Read RABBIT data for barley.
genoFile <- system.file("extdata/barley", "barley_magicReconstruct.zip",
                       package = "statgenMPP")
barleyMPMPP <- readRABBITMPP(unzip(genoFile, exdir = tempdir()))
}

}
\references{
Zheng, Chaozhi, Martin P Boer, and Fred A Van Eeuwijk.
“Recursive Algorithms for Modeling Genomic Ancestral Origins in a Fixed
Pedigree.” G3 Genes|Genomes|Genetics 8 (10): 3231–45.
https://doi.org/10.1534/G3.118.200340.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{maizeSQM}
\alias{maizeSQM}
\title{Pre-computed SQM output maize}
\format{
An object of class \code{QTLMPP} (inherits from \code{GWAS}) of length 5.
}
\usage{
maizeSQM
}
\description{
Pre-computed SQM output for maize data used in vignette.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createQTLMPP.R
\name{plot.QTLMPP}
\alias{plot.QTLMPP}
\title{Plot function for the class \code{QTLMPP}}
\usage{
\method{plot}{QTLMPP}(
  x,
  ...,
  plotType = c("QTLProfile", "parEffs", "QTLRegion", "QTLProfileExt", "parCIs"),
  title = NULL,
  output = TRUE
)
}
\arguments{
\item{x}{An object of class \code{QTLMPP}.}

\item{...}{further arguments to be passed on to the actual plotting
functions.}

\item{plotType}{A character string indicating the type of plot to be made.
One of "QTLProfile", "parEffs", and "QTLRegion".}

\item{title}{A character string, the title of the plot.}

\item{output}{Should the plot be output to the current device? If
\code{FALSE}, only a ggplot object is invisibly returned.}
}
\description{
Creates a plot of an object of S3 class \code{QTLMPP}. The following types of
plot can be made:
\describe{
\item{QTLProfile}{A QTL profile plot, i.e. a plot of \eqn{-log10(p)} values
per marker.}
\item{parEffs}{A plot of effect sizes and directions per parent.}
\item{QTLRegion}{A plot highlighting the QTLs found on the genetic map.}
\item{QTLProfileExt}{A combination of the QTL profile and parental effects
plotted above each other.}
\item{parCIs}{A plot of the effect sizes and confidence intervals for each
parent for each of the QTLs found.}
}
See details for a detailed description of the plots and the plot options
specific to the different plots.
}
\section{QTL Profile plot}{

A profile of all marker positions and corresponding \eqn{-log10(p)} values is
plotted. QTLs found are highlighted with red dots. The threshold is plotted
as a horizontal line. If there are previously known marker effects, false
positives and true negatives can also be marked.\cr
Extra parameter options:
\describe{
\item{\code{xLab}}{A character string, the x-axis label. Default =
\code{"Chromosomes"}}
\item{\code{yLab}}{A character string, the y-axis label. Default =
\code{-log10(p)}}
\item{\code{effects}}{A character vector, indicating which markers correspond
to a real (known) effect. Used for determining true/false positives and
false negatives. True positives are colored green, false positives orange and
false negatives yellow.}
\item{\code{colPalette}}{A color palette used for plotting. Default
coloring is done by chromosome, using black and grey.}
\item{\code{signLwd}}{A numerical value giving the thickness of the
points that are false/true positives/negatives. Default = 0.6}
\item{\code{chr}}{A vector of chromosomes to be plotted. By default, all
chromosomes are plotted. Using this option allows restricting the plot to a
subset of chromosomes.}
}
}

\section{Parental effects Plot}{

A plot of effect sizes for each of the parents for the QTLs found is
created.\cr
Extra parameter options:
\describe{
\item{\code{xLab}}{A character string, the x-axis label. Default =
\code{"Chromosomes"}}
\item{\code{yLab}}{A character string, the y-axis label. Default =
\code{"Parents"}}
\item{\code{chr}}{A vector of chromosomes to be plotted. By default, all
chromosomes are plotted. Using this option allows restricting the plot to a
subset of chromosomes.}
}
}

\section{QTL Region Plot}{

A plot of effect is created on which the positions of the QTLs found in
the QTL detection are highlighted in red.\cr
No extra parameter options.
}

\section{Extended QTL Profile Plot}{

An extended version of the QTL Profile Plot, in which the QLT profile plot is
combined with the parental effect plot to make it easier to assess the
effects for each specific QTL found.\cr
Extra parameter options:
\describe{
\item{\code{chr}}{A vector of chromosomes to be plotted. By default, all
chromosomes are plotted. Using this option allows restricting the plot to a
subset of chromosomes.}
}
}

\section{Parental Confidence Interval Plot}{

A plot of the parental effects and the confidence intervals around them for
each of the QTLs found. Positive significant effects are shown in red,
negative significant effects in blue, and all other effects are shown in
black.
}

\examples{
\dontrun{
## Read phenotypic data.
pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                               package = "statgenMPP"))
## Rename first column to genotype.
colnames(pheno)[1] <- "genotype"


## Compute IBD probabilities for simulated population - AxB, AxC.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = c(system.file("extdata/multipop", "AxB.txt",
                                              package = "statgenMPP"),
                                  system.file("extdata/multipop", "AxC.txt",
                                              package = "statgenMPP")),
                  pheno = pheno,
                  popType = "F4DH",
                  mapFile = system.file("extdata/multipop", "mapfile.txt",
                                        package = "statgenMPP"),
                  evalDist = 5)

## Multi-QTL Mapping.
ABC_MQM <- selQTLMPP(ABC, trait = "yield")

## QTL Profile plot.
plot(ABC_MQM, plotType = "QTLProfile")

## Plot of parental effects for QTLs found.
plot(ABC_MQM, plotType = "parEffs")

## Plot of genetic map highlighting positions of QTLs found.
plot(ABC_MQM, plotType = "QTLRegion")

## Extended QTL Profile plot.
plot(ABC_MQM, plotType = "QTLProfileExt")

## Parental confidence interval plot.
plot(ABC_MQM, plotType = "parCIs")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/randomQTLmodel.R
\name{randomQTLmodel}
\alias{randomQTLmodel}
\title{one position fitted in the mixed model}
\usage{
randomQTLmodel(
  modDat,
  map,
  parents,
  trait = "yield",
  scanMrk = NULL,
  cofMrk = NULL,
  NULLmodel = FALSE
)
}
\description{
one position fitted in the mixed model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createGDataMPP.R
\name{createGDataMPP}
\alias{createGDataMPP}
\title{Create an object of class gDataMPP}
\usage{
createGDataMPP(IBDprob, pheno)
}
\arguments{
\item{IBDprob}{An object of class \code{IBDprob}.}

\item{pheno}{A data frame with at least columns "genotype" for the
"genotype" and one or more numerical columns containing phenotypic
information. A column "cross" can be used for indicating the cross the
genotype comes from. This column is ignored if the \code{IBDprob} has an
attribute genoCross containing this information (the default behaviour).}
}
\value{
An object of class \code{gDataMPP}
}
\description{
Function for creating an object of class gDataMPP from an object of class
IBDprob (computed or imported using statgenIBD) and phenotypic data.
}
\examples{
## Read phenotypic data.
pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                               package = "statgenMPP"))
## Rename first column to genotype.
colnames(pheno)[1] <- "genotype"

## Compute IBD probabilities for simulated population using statgenIBD - AxB
AB <- statgenIBD::calcIBD(popType = "F4DH",
                         markerFile = system.file("extdata/multipop", "AxB.txt",
                                                 package = "statgenMPP"),
                         mapFile = system.file("extdata/multipop", "mapfile.txt",
                                              package = "statgenMPP"),
                         evalDist = 5)

## Create an object of class gDataMPP from IBD probabilities and
## phenotypic data.
ABMPP <- createGDataMPP(IBDprob = AB, pheno = pheno)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selQTLMPP.R
\name{selQTLMPP}
\alias{selQTLMPP}
\title{Multi round genome scans for QTL detection}
\usage{
selQTLMPP(
  MPPobj,
  trait = NULL,
  QTLwindow = 10,
  threshold = 3,
  maxCofactors = NULL,
  K = NULL,
  computeKin = FALSE,
  parallel = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{MPPobj}{An object of class gDataMPP, typically the output of either
\code{\link{calcIBDMPP}} or \code{\link{readRABBITMPP}}.}

\item{trait}{A character string indicating the trait QTL mapping is done for.}

\item{QTLwindow}{A numerical value indicating the window around a QTL that
is considered as part of that QTL.}

\item{threshold}{A numerical value indicating the threshold for the
\eqn{-log10p} value of a marker to be considered a QTL.}

\item{maxCofactors}{A numerical value, the maximum number of cofactors to
include in the model. If \code{NULL} cofactors are added until no new
cofactors are found.}

\item{K}{A list of chromosome specific kinship matrices. If
\code{NULL} and \code{computeKin = FALSE} no kinship matrix is included in
the models.}

\item{computeKin}{Should chromosome specific kinship matrices be computed?}

\item{parallel}{Should the computation of variance components be done in
parallel? This requires a parallel back-end to be registered. See examples.}

\item{verbose}{Should progress and intermediate plots be output?}
}
\value{
An object of class \code{QTLMPP}
}
\description{
Multi round genome scans for QTL detection.\cr\cr
Several rounds of QTL detection are performed. First a model is fitted
without cofactors. If for at least one marker the \eqn{-log10(p)} value is
above the threshold the marker with the lowest p-Value is added as cofactor
in the next round of QTL detection. This process continues until there are
no new markers with a \eqn{-log10(p)} value above the threshold or until
the maximum number of cofactors is reached.
}
\details{
By default only family specific effects and residual variances and no
kinship relations are included in the model. It is possible to include
kinship relations by either specifying \code{computeKin = TRUE}. When doing
so the kinship matrix is computed by averaging \eqn{Z Z^t} over all markers,
where \eqn{Z} is the genotype x parents matrix for the marker. It is also
possible to specify a list of precomputed chromosome
specific kinship matrices in \code{K}. Note that adding a kinship matrix
to the model increases the computation time a lot, especially for large
populations.
}
\examples{
## Read phenotypic data.
pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                               package = "statgenMPP"))
## Rename first column to genotype.
colnames(pheno)[1] <- "genotype"


## Compute IBD probabilities for simulated population - AxB, AxC.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = c(system.file("extdata/multipop", "AxB.txt",
                                              package = "statgenMPP"),
                                  system.file("extdata/multipop", "AxC.txt",
                                              package = "statgenMPP")),
                  pheno = pheno,
                  popType = "F4DH",
                  mapFile = system.file("extdata/multipop", "mapfile.txt",
                                        package = "statgenMPP"),
                  evalDist = 5)

## Single-QTL Mapping.
ABC_SQM <- selQTLMPP(ABC, trait = "yield", maxCofactors = 0)

## Multi-QTL Mapping.
\dontrun{
## Register parallel back-end with 2 cores.
doParallel::registerDoParallel(cores = 2)

## Run multi-QTL mapping.
ABC_MQM <- selQTLMPP(ABC, trait = "yield", parallel = TRUE)

## Run multi-QTL mapping - include kinship matrix.
ABC_MQM_kin <- selQTLMPP(ABC, trait = "yield", parallel = TRUE,
                        computeKin = TRUE)
}

}
\seealso{
\code{\link{kinshipIBD}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcIBDMPP.R
\name{calcIBDMPP}
\alias{calcIBDMPP}
\title{IBD calculation for multi parental populations}
\usage{
calcIBDMPP(
  crossNames,
  markerFiles,
  pheno,
  popType,
  mapFile,
  evalDist,
  grid = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{crossNames}{A character vector, the names of the crosses.}

\item{markerFiles}{A character vector indicating the locations of the files
with genotypic information for the populations. The files should be in
tab-delimited format with a header containing marker names.}

\item{pheno}{A data.frame or a list of data.frames with phenotypic data,
with genotypes in the first column \code{genotype} and traits in the
following columns. The trait columns should be numerical columns only.
A list of data.frames can be used for replications, i.e. different
trials.}

\item{popType}{A character string indicating the type of population. One of
DH, Fx, FxDH, BCx, BCxDH, BC1Sx, BC1SxDH, C3, C3DH, C3Sx, C3SxDH, C4, C4DH,
C4Sx, C4SxDH (see Details).}

\item{mapFile}{A character string indicating the location of the map file
for the population. The file should be in tab-delimited format. It should
consist of exactly three columns, marker, chromosome and position. There
should be no header. The positions in the file should be in centimorgan.}

\item{evalDist}{A numeric value, the maximum distance in cM between
evaluation points.}

\item{grid}{Should the extra markers that are added to assure the a
maximum distince of \code{evalDist} be on a grid (\code{TRUE}) or in between
marker existing marker positions (\code{FALSE}).}

\item{verbose}{Should progress be printed?}
}
\value{
An object of class \code{gDataMPP} with the following components:
\item{\code{map}}{a data.frame containing map data. Map is sorted by
chromosome and position.}
\item{\code{markers}}{a 3D matrix containing IBD probabilities.}
\item{\code{pheno}}{data.frame or list of data.frames containing phenotypic
data.}
\item{\code{kinship}}{a kinship matrix.}
\item{\code{covar}}{a data.frame with extra covariates (including the
name of the cross).}
}
\description{
IBD calculation for multi parental populations. Per cross IBD probabilities
are calculated using \code{calcIBD} in the statgenIBD package. These
probabilities are combined with optional phenotypic data and stored in a
single object of class \code{gDataMPP}.
}
\details{
IBD probabilities can be calculated for many different types of populations.
In the following table all supported populations are listed. Note that the
value of x in the population types is variable, with its maximum value
depicted in the last column.\tabular{llll}{
   \strong{Population type} \tab \strong{Cross} \tab \strong{Description} \tab \strong{max. x} \cr
   DH \tab biparental \tab doubled haploid population \tab  \cr
   Fx \tab biparental \tab Fx population (F1, followed by x-1 generations of selfing) \tab 8 \cr
   FxDH \tab biparental \tab Fx, followed by DH generation \tab 8 \cr
   BCx \tab biparental \tab backcross, second parent is recurrent parent \tab 9 \cr
   BCxDH \tab biparental \tab BCx, followed by DH generation \tab 9 \cr
   BC1Sx \tab biparental \tab BC1, followed by x generations of selfing \tab 7 \cr
   BC1SxDH \tab biparental \tab BC1, followed by x generations of selfing and DH \tab 6 \cr
   C3 \tab three-way \tab three way cross: (AxB) x C \tab  \cr
   C3DH \tab three-way \tab C3, followed by DH generation \tab  \cr
   C3Sx \tab three-way \tab C3, followed by x generations of selfing \tab 7 \cr
   C3SxDH \tab three-way \tab C3, followed by x generations of selfing and DH generation \tab 6 \cr
   C4 \tab four-way \tab four-way cross: (AxB) x (CxD) \tab  \cr
   C4DH \tab four-way \tab C4, followed by DH generation \tab  \cr
   C4Sx \tab four-way \tab C4, followed by x generations of selfing \tab 6 \cr
   C4SxDH \tab four-way \tab C4, followed by x generations of selfing and DH generation \tab 6 \cr
}
}
\examples{
## Read phenotypic data.
pheno <- read.delim(system.file("extdata/multipop", "AxBxCpheno.txt",
                               package = "statgenMPP"))
## Rename first column to genotype.
colnames(pheno)[1] <- "genotype"


## Compute IBD probabilities for simulated population - AxB, AxC.
ABC <- calcIBDMPP(crossNames = c("AxB", "AxC"),
                  markerFiles = c(system.file("extdata/multipop", "AxB.txt",
                                              package = "statgenMPP"),
                                  system.file("extdata/multipop", "AxC.txt",
                                              package = "statgenMPP")),
                  pheno = pheno,
                  popType = "F4DH",
                  mapFile = system.file("extdata/multipop", "mapfile.txt",
                                        package = "statgenMPP"),
                  evalDist = 5)

summary(ABC)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parCIsPlot.R
\name{parCIsPlot}
\alias{parCIsPlot}
\title{Create a plot with effect sizes}
\usage{
parCIsPlot(
  parCIsDat,
  xLab = "Effect",
  yLab = "Parents",
  title = NULL,
  trait = NULL,
  ...,
  output = TRUE
)
}
\description{
Create a plot of QTL effect sizes and standard errors for each parent for
selected snps.
}
\keyword{internal}
